//BA44_ReActs.ino

char thisrobot[] ="Bambino ";
char thisprog[]  ="BA44_ReActs"; 

// Sketch uses 18818 bytes (58%) of program storage space. Maximum is 32256 bytes.
// Global variables use 838 bytes (40%) of dynamic memory, leaving 1210 bytes for local variables. Maximum is 2048 bytes.

//set Serial terminal to 9600 baud
//== #define ================================================
//#define BambinoPM         //ProMini 328,5v
//#define BambinoMM         //Uno with no Whisker sensors
#define BambinoM2         //Uno 

/*
== Notes ====================================================
Previously Reactions were hard coded in the program which meant 
they couldn't be altered as easily as writing a new Act.
Behaviour parameters are stored in EEPROM page 0, 
Acts are stored in EEPROM pages 1 and 2, Acts in page 1 and 
Acts0 in page 2
Which leaves EEPROM page 3 spare.
Reactions will now be stored in EEPROM page 3 which will allow 
easy experimentation with different Reactions.
There are 8 slots available and they are allocated as
  Reaction-1 - Reaction if Ultrasonic distance is less than 
               'aware at' value, hence objects farther away 
               than 'aware at' value are ignored
  Reaction-2 - Reaction if Ultrasonic distance is less than 
               'too close' value, i.e. objects are too close
  Reaction-3 - Reaction if both Left and Right Whiskers sense
               an object
  Reaction-4 - Reaction if Left Whisker senses an object
  Reaction-5 - Reaction if Right Whisker senses an object 
  Reaction-6 - Reaction if both Left and Right Beard sensors 
               detect a hole
  Reaction-7 - Reaction if Left Beard sensor detects a hole 
  Reaction-8 - Reaction if Right Beard sensor detects a hole 
  
Introducing
  ! to change to EEPROM page 3, Reactions, in the same way 
    as 0 changes to Acts page 0.
  Reactions can be recorded or loaded just like Acts.
    !r1 - records Reaction-1 etc.
    !1  - plays Reaction-1 etc
    l!1 or !l1 - loads Reaction-1 etc 
  bum+ - set manually SonarAwareAt, units 50mm/2inch
  bum- - set manually SonarTooNear, units 50mm/2inch
  bu+,0-9 - set SonarAwareAt, units 50mm, distance beyond 
            which objects are ignored   
  bu-,0-9 - set SonarTooNear, units 50mm, distance at which 
            objects are too near
  bus - save current Sonar settings to EEPROM  
  
  The commansa 'U','W','G' now read the sensors and react to 
  objects, obstacles and holes using ReActions in EEPROM.

============================================================
Note - The Arduino Serial Monitor can't send [Esc]. 
       If another Terminal is used which can send [Esc] then 
       at GetCcmnd() this will be printed, which may print 
       as a symbol or the Terminal may change it to "."
     
Bugs -	   
============================================================
improvements? - 
- GetAct should use GetCcmnd to read Act from current act rather than a prompt. rx,lx,cx would then be playable
- void GetCcmnd()	  could use   case 0:   - why?

To do next - 
b,1-8 - set behaviour slot and copy contents to behaves
bc,1-8,1-8 - copy one behaviour slot to another
bl,1-8 - load a behaviour slot, <char>,...,#<decimal byte value>, etc. otherwise send all byte values/chars.
bs,1-8 - save current behaviour to a slot
?B - print all behaviour slots as a table
?b - as before, print behaves.

/R ! change to Reactions page1 0 changes to Rpage0   8x16 byte slots in each Rpage 
                             Active page set in active BeHaves   
/A change to Acts Apage1     0 changes to Apage0   8x32 byte slots in each Apage
/B change to BeHaviour page  8x16 byte slots, load only, /B1 - /B8

Ername, Ergroup, Default BeHaviour - need to be  separate
   
print 23 to 31 for 'm' machine
bm 31 Ef_walking =255;
bm 30 ErollC     =254;    -> (90) rollC   set in initPM34.h from EEPROM set in initialiseEEPROM()
bm 29 EpaceC     =253;    -> (90) paceC   set in initPM34.h from EEPROM set in initialiseEEPROM()
bm 28 Erollby    =252;    -> (15) rollby  set in initPM34.h from EEPROM set in initialiseEEPROM()   physical max 20, [br,1-9]
bm 27 Epaceby    =251;    -> (15) paceby  set in initPM34.h from EEPROM set in initialiseEEPROM()  physical max 20, [bp,1-9] 
   26 [Erollmin   =250;]
   25 [Erollmax   =249;]
   24 [Epacemin   =248;]
   23 [Epacemax   =247;]
bm 22 Ergroup     =246;   // [b#,A-Z]
bm 21 Ername    =245;     // [b#,0-9]
bm 20 EautoAct   =244;    // [ba,0-8]
bm 19 ESonarAwareAt =243  // [bu(m)+,1-9]
bm 18 ESonarTooNear =242  // [bu(m)-.1-9]
[ boolean f_cm ]
print 0 to here plus 31 for Mind
  *17 LeftRightLightTurnThresholdScale - 1-9 in units of 1%
  *16 LeftRightLightTurnThreshold      - 0-9 in units of Scale
  *15 holeThresholdScale     - 1-9 in units of 1%
  *14 holeThresholdRight     =10; % - 0-9  units of 5%  // difference between RbeardLight and RbeardDark for a hole
  *13 holeThresholdLeft      =10; % - 0-9  units of Scale // difference between LbeardLight and LbeardDark for a hole 
  *12 obstacleThresholdScale - 0-9  units of 1%  
  *11 obstacleThresholdRight  =20; % - 0-9  units of 5%  // difference between RwhiskerLight and RwhiskerDark for an obstacle
  *10 obstacleThresholdLeft   =20; % - 0-9  units of 5%  
   9 SonarAwareAt - 5cm units - 1 to 9 = 5cm to 45cm
   8 SonarTooNear - 5cm units - 1 to 9 = 5cm to 45cm
bm 7 rollspeedmax [sm 1-9]    (4)  // quite fast 
bm 6 pacespeedmax [sm 1-9]    (6)  // quite fast 
bm 5 rollspeed    [s,sr 1-9]  (1) slow
bm 4 pacespeed    [s,sp 1-9]  (1) slow
bm 3 turnby       [bt 0-9 *2] (paceby) units of 10us  set in initPM34.h from EEPROM set in initialiseEEPROM()   
bm 2 rollby       [br 0-9 *2] (15) units of 10us set in initPM34.h from EEPROM set in initialiseEEPROM()  physical max 20
bm 1 paceby       [bp 0-9 *2] (15) units of 10us set in initPM34.h from EEPROM set in initialiseEEPROM()  physical max 20 
 0 
  
bp(1-8) - print Behaviour in EEPROM (1-8)
     all moveby values are 2* command values, only print command values
   - f_walking, rollC, paceC, rollby, paceby, rollmin, rollmax, pacemin, pacemax
lb(1-8) - load b(1-8)  Load Mind  - EEPROM values    - byte values - prefix with #
b(1-8) - load Mind (RAM values) with behaviour from EEPROM (1-8)
b8 - default behaviour loaded to Mind (RAM) in Init
Cr+,Cr-(1-9) - alter rollC in RAM and EEPROM in current mind
Cp+,Cp-(1-9) - alter paceC in RAM and EEPROM in current mind 
u+(1-8) - ultrasonic furthest range aware at, further ignored
u-(1-8) - ultrasonic closest range, less is too close

b0 - reinitialise behaves - do a software restart using the watchdog
b<1-8> - select current behaviour

Superceded by ReActs
16 bytes for behaves => 8 in 128 bytes
sr,sp,srm,spm,mr,mp,mt,u-,u+,(spare 7)
two bytes each => 16 bytes, => 8 in 128 bytes
bG-<{0},1-8> - Act to be done on hole-tests left & right true 
bGR<{0},1-8> - Act to be done on hole-test right true 
bGL<{0},1-8> - Act to be done on hole-test left  true 
bW-<{0},1-8> - Act to be done on Whisker-tests left & right true 
bWR<{0},1-8> - Act to be done on Whisker-test right true 
bWL<{0},1-8> - Act to be done on Whisker-test left  true
bU+<{0},1-8> - Act to be done on Ultrasonic distance <= aware at
bU-<{0},1-8> - Act to be done on Ultrasonic distance <= too near
=============================================================
Primary commands used
012345678 ?#. ABEFGHLRSUVWX bclrstvw
   
COMMANDS
======== 
'.',[Esc] - quit current operation - recording, playing, input
'S' - Stop/Start
'!' - use page3 Reacts for the next record, load, replay, or print
'0' - use page0 for the next Act
'1'-'8' the routine or Act to record or replay
'F' - Forwards one step
'B' - Backwards one step
'R' - turn Right one step
'L' - turn Left one step
'H' - Halt with both feet together
'V' - Voice, make a beep, whee
'U' - read Ultrasonic sensor and react to objectRange using reactUObjects() and ReActs in EEPROM
'W' - read Whisker sensors and avoid holes (if any) using reactObstacles() and ReActs in EEPROM
'G' - read Beard sensors and avoid obstacles (if any) using reactHoles() and ReActs in EEPROM
'A' - beAlive()  until '.' or Esc or reset      // the loop() from B_PM22_BeAlive
'X' - Xplore()   until '.' or Esc or reset      // Wander()
'E' - Experiment - user routine  until '.' or Esc or reset
'v', 'b','e','o' choose sound: 'b' bip, 'e' whee, 'o' whoa
'w', '1'-'9' - wait 1 - 9 seconds 
'su' - speedup, speed is increased by 1 up to max
'sd' - speed down, speed is divided by 2 down to 1
's','1'-'9' set speed, both roll and pace speeds set to the same
'sr','1'-'9' set roll speed                              units of servo degrees change per frame
'sp','1'-'9' set pace speed                              units of servo degrees change per frame
'sm' - set rollspeedmax to current rollspeed, set pacespeedmax to current pacespeed
'srm' - set rollspeedmax to current rollspeed
'spm' - set pacespeedmax to current pacespeed
'b','#','A'-'z' - robot name  A-z
        '1'-'9' - robot group 1-9
    'a','0'-'8' - set Autorun act '1' - '8', '0' no Autorun act   
    'a','9'     - set Autorun act to run when there is nothing else to do, ## turns it off 
                  not remembered through reset/power-off - default off  
    'q' - Quiet - turn off status messages
    'w0' - turn off walking 
    'w1' - turn on walking
    'p','1'-'9' - set paceby - step length                units of 2 servo degrees
    't','1'-'9' - set turnby - step length in turns       units of 2 servo degrees
    'r','1'-'9' - set rollby - roll amount when stepping  units of 2 servo degrees
    'um+' - set manually SonarAwareAt, units 50mm
    'um-' - set manually SonarTooNear, units 50mm
    'u+','0'-'9' - set SonarAwareAt, units 50mm    
    'u-','0'-'9' - set SonarTooNear, units 50mm    
    'v','0','1' - verbose, light level reporting of sensors, 0=off, 1=on
    'us' - save current Sonar settings to EEPROM
    '@' - Adjust Servos 
't','a' - turn Away from leading foot, will be used in avoid Acts
    'm' - turn Memory (same way as last turn, will be used in avoid Acts
    'o' - turn Other way from last time, will be used in avoid Acts 
    'l' - turn to light
    'd' - turn to dark
    '@l' - turn until @ light
    '@d' - turn until @ dark    
'r','1'-'8' - Record moves into selected Act in EEPROM
'c','1'-'8','1'-'8' -  copy one Act to another Act, xy,x0y,0xy,0x0y
'l','1'-'8' - load Act, like 'r' but records commands into EEPROM without playing them
'?','h' - print Help instructions
    '1'-'8' - Print moves in selected Act '1' - '8'
    'p' - print current Page
    '?' - print current Page
    'd' - Dump out all acts to the PC, same as 'p' but without the Act and moves numbers.
    'b' - print Behaviour - behaviour variables in RAM
    'm' - print Mind - all Acts, Behaviour
    '!' - print ReActs
'#','*' - message for all robots, robots silent
    '0-9' - message for robot group 0-9, robots silent
    'A-z' - message for robot name A- 9 in current group, other robots silent
    '#' - (default) robots stop, obey all commands and send messages     
          useful to talk to an unknown robot or put all in the stopped state 
    '<' - do Default Act   
    '>' - resume Act   
    '?' - all robots reply with their name and group - delayed according to name
   
The whiskerLEDs glowing dim indicate Bambino is ready for a command.
If other keys are pressed, including lower case keys, then the Whisker LEDs will flash badkey().
(MM - mouth glows dim while ready and Beard LEDs used for badkey().) 

RESPONSES
sense44.ino - readLight()  // read ambient Light levels   0 - 1023
  [a<LeyeLight>|<ReyeLight>]
sense44.ino - readWhiskers()
  [W<LwhiskerLight>,<LwhiskerDark>,<LwhiskerLight -LwhiskerDark>|<RwhiskerLight>,<RwhiskerDark>,<RwhiskerLight -RwhiskerDark>]
sense44.ino - readBeard()
  [B<LbeardLight>,<LbeardDark>,<LbeardLight -LbeardDark>|<RbeardLight>,<RbeardDark>,<RbeardLight -RbeardDark>]
sense44.ino - showObstacles()
  [o%<f_obstacleLeft><f_obstacleRight>]
sense44.ino - showHoles()
  [h%<f_holeLeft><f_holeRight>]
=============================================================
ReActs
======
Reactions stored in EEPROM page 3 as 8 slots of 32 bytes, Reactions can have the same commands as Acts and ended with ".".
When a sensor test is made in an Act and something is detected then the relevant Reaction is played then the Act continues.
  Reaction-1 - Reaction if Ultrasonic distance is less than 'aware at' value  
                 hence objects farther away than 'aware at' value are ignored
  Reaction-2 - Reaction if Ultrasonic distance is less than 'too close' value
                 i.e. objects are too close
  Reaction-3 - Reaction if both Left and Right Whiskers sense an object
  Reaction-4 - Reaction if Left Whisker senses an object
  Reaction-5 - Reaction if Right Whisker senses an object 
  Reaction-6 - Reaction if both Left and Right Beard sensors detect a hole
  Reaction-7 - Reaction if Left Beard sensor detects a hole 
  Reaction-8 - Reaction if Right Beard sensor detects a hole 
=============================================================
BEHAVES - power-on defaults
=======
sense behaves
  SonarAwareAt  is EEPROM value  
  SonarTooNear  is EEPROM value
speed behaves  - set in void setup()  
  rollspeed     is 1             
  pacespeed     is 1
  rollspeedmax  is 4
  pacespeedmax  is 6
move behaves   - set in void setup()  init42.h 
  rollby   is EEPROM value,    set by B_PM04_Servos or AdjustServos()
  paceby   is EEPROM value,    set by B_PM04_Servos or AdjustServos()
  turnby   is paceby
varsCons44.h
  rollC =90;    default, AdjustServos39.ino, roll centre = 90 degrees (ie 1500us) => upright
  paceC =90;    default, AdjustServos39.ino, pace centre = 90 degrees (ie 1500us) => feet together  
  Servoframe =20; never changed, Servo frame rate millisecs
//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pins20.h     - pin usage
//     varsCons44.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h       - the Arduino EEPROM library
//     avr/pgmspace.h - to allow the printHelp() text to be in ProgramSpace
//== VARIABLES and CONSTANTS 
//== HELP TEXT 
//     PROGMEM
//== SETUP
//     init44.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   GetCcmnd()	       //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
//   RecordCcmnd()	
//   Experiment()       //until Quit
//   Xplore()         //Wander() until Quit
//   makeCcmndDigit()  ensure 1-9
//   waitCcmndSecs()   //read delay time from Act and wait
//   printMind()
//   PrintReActs()
//   printReAct()
//   printBehaviour()
//   DumpActs()        //Dump Acts, Print without names and length
//   loadAct()
//   copyAct()
//   Ccmnd <- PlayAct() 
//   PrintAllActs()
//   printAct()
//   Act <- getAct()  //called when recording, loading, printing and copying Acts 
//   setEEPROMindexes(char Act)      //  changes EEPROMindex, ActEnd
//   badkey()         // indicate invalid key
//   printHelp()
//   USreact()        // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
//   WhiskerReact()   // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
//   BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
//   beAlive()        // the loop() from BA22_BeAlive
//   redLEDdim() - BambinoMM only
//   redLEDon()
//   redLEDoff() 
//   bip()            // make a bip sound
//   whiskerLEDsDim() - not BambinoMM
//   whiskerLEDsOff() - not BambinoMM
//TAB FILES
//   AdjustServos39.ino
//     AdjustServos(), printEEPROMvalues(), printAdjustInstructions() 
//     PROGMEM adjust_instructions
//   acts40.ino
//     ACTS - WanderSteps(steps), Wander()
//            RightStep(steps), LeftStep(steps)
//            rock(repeats), shuffle(repeats)
//     SOUND -  whee(), whoa(), beep(byte beeppin,int beepfreqHz,long beeptime)  
//   behaviour44.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - reactUObjects(), reactObstaclesLR(), reactHolesLR()
//                     - avoidUObjects(), avoidObstaclesLR(), avoidHolesLR()
//   sense44.ino
//     SENSE - readUSdistance(), showUSdistance(), dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels(), showObstacles() 
//           - readBeard(), QueryEdgeLR(), showBeardLightLevels(), void showHoles()
//           - readLight()
//   movement34.ino
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(), 
//                halt(), 
//                rollCentre(), rollLeft(), rollRight(),
//                paceCentre(), paceLeftForward(),  paceRightForward(), 
//                              paceLeftBackward(), paceRightBackward(), 
//                              turnLeftForward(),  turnRightForward(), 
//                              turnLeftBackward(), turnRightBackward(), 
//                rollmove(), pacemove()
//   skills41.ino
//     SKILLS - turnLight(), turnDark(), turnAtLight(), turnAtDark()
//     META MOVEMENT - turnAway(), turnMemory(), turnOther()  
//   servos40.ino
//     SERVOS - ServosInitC()
//              ServoPulse(byte Spin, int Spulse)
//   system43.ino
//     SYSTEM - disable_walking(), enable_walking(), toggle_f_walking(), initialiseEEPROM()
//===========================================================
*/

/*== NEW ====================================================
varsCons44.h
  //EEPROM addresses
  const byte ESonarAwareAt =243  // 1-9  init(5) -> SonarAwareAt
  const byte ESonarTooNear =242  // 1-9  init(2) -> SonarTooNear
  //BEHAVIOUR
  byte SonarTooNear ;   // init =EEPROM(2), 'bu(m)-', units 50mm/2inch, objects nearer will be considered as Too Near and avoided  
  byte SonarAwareAt ;   // init =EEPROM(5), 'bu(m)+', units 50mm/2inch, objects nearer will avoided, objects farther away will be ignored
  //MIND
  boolean f_bv =0;          //verbose, light level reporting of sensors, 0=off, 1=on
boolean f_reacting =false;    //in a ReAct - set in behaviour44.ino if a React is initiated.
int ReactAt;                  //copy of EEPROMindex at React
void PrintReActs()   
  { for(Act ='1'; Act<='8'; Act++) { epage =3; printAct(); }  Serial.println(); }
void beardLEDsOff() 
sense42.ino -> sense44.ino   showObstacles(), void showHoles()
/*== CHANGED ================================================
hardware20.ino -> hardware44.ino
varsCons42.h -> varsCons44.h   Unear Ufar
void loop()
  if(!f_reacting)                    // don't change if in a ReAct
    { if(f_play)
        { whiskerLEDsOff();  }
        else
        {  whiskerLEDsDim(); }    //indicate ready for command
      }  
  if(!f_play && !f_reacting && !f_BeQuiet){Serial.print(">"); }    //ready prompt
    case '.':   if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }   //terminated string
                if(f_reacting) { f_reacting =false; EEPROMindex =ReactAt; beardLEDsOff(); }  
                            else { f_play =false; }  
    case 'b'
                  case 'u': if (f_recording) { RecordCcmnd(); }    //so 'u' is recorded
                            GetCcmnd();                             //get parameter
                            switch (Ccmnd)
                            { case 'm': // don't record 'm'
                                        do
                                        { Serial.println("press [+] for AwareAt, [-] for TooNear, [.] or [Esc] to Quit")
                                          GetCcmnd();                            //get parameter
                                          objectRange =dist_HC_SR04(); if(f_cm) {objectRange /=5;} else {objectRange /=2;}
                                          if (Ccmnd=='+') 
                                            { SonarAwareAt =objectRange; 
                                              if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                              Ccmnd ='0' +SonarAwareAt; if (f_recording) { RecordCcmnd(); }    //so 'SonarAwareAt' is recorded
                                              }
                                          if (Ccmnd=='-')
                                            { SonarTooNear =objectRange;
                                              if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                              Ccmnd ='0' +SonarTooNear; if (f_recording) { RecordCcmnd(); }    //so 'SonarTooNear' is recorded
                                              }
                                        } while (Ccmnd!='.' &&  Ccmnd!='Q' && Ccmnd!=Esc)
                                        break;
                              case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarAwareAt =Ccmnd -'0';
                                        break;
                              case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarTooNear =Ccmnd -'0';
                                        break;
                              case 's': EEPROM.write(ESonarAwareAt,SonarAwareAt);          
                                        EEPROM.write(ESonarTooNear,SonarTooNear);          
                                        break;                                        
                            }     // end switch
                            break;
boolean Quit()          //check for Quit
    if( Ccmnd=='.' || Ccmnd==Esc || Ccmnd=='#')  
char PlayAct() 
   deleted if(_Ccmnd=='.') { f_play =false; }         //play is unset in case '.'
void printBehaviour()
  Serial.print(SonarAwareAt);  Serial.println(" SonarAwareAt");   
  Serial.print(SonarTooNear);  Serial.println("SonarTooNear");
void USreact()     // added units cm, inch
  if( !f_BeQuiet ) { Serial.print(objectRange); }
  if( !f_BeQuiet ) { if(f_cm) {Serial.println("cm");} else {Serial.println("inch");}
  //changed call from avoidUObjects() to reactUObjects()
void WhiskerReact()    
  //changed call from avoidObstaclesLR() to reactObstaclesLR()
void BeardReact()   
  //changed call from avoidHolesLR() to reactHolesLR()
behaviour20.ino -> behaviour44.ino
  void reactUObjects() 
  void reactObstaclesLR()
  void reactHolesLR()    
 
//== HELP TEXT
const char h_bump[] PROGMEM = "    'um+' - manually set SonarAwareAt";
const char h_bumn[] PROGMEM = "    'um-' - manually set SonarTooNear";
const char h_bup[]  PROGMEM = "    'u+','1'-'9' - set SonarAwareAt";
const char h_bun[]  PROGMEM = "    'u-','1'-'9' - set SonarTooNear";
const char h_bus[]  PROGMEM = "    'us' - save current Sonar settings to EEPROM";
*/
//== INCLUDES ===============================================
#include "pins20.h"
#include "varsCons44.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
#include <avr/pgmspace.h>     // for PROGMEM for the printHelp() text
  
//== VARIABLES and CONSTANTS ================================
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play current Act
boolean f_reacting =false;    //in a ReAct - set in behaviour44.ino if a React is initiated.
boolean f_doDefault =true;    //true for the first loop and #< only
boolean f_runDefault =false;  //run Default when nothing else to do
const byte NL =10;            //NewLine value
const byte LF =10;            //LineFeed value                                  
const byte CR =13;            //Carriage Return value
const byte Esc =27;           //Esc key value
char Act;                     //current Act as '1' - '8'
byte actIndex =0;             //current Act number, set to default           
int EEPROMindex =-1;          //index to moves in an Act in EEPROM, -1 => nothing to continue with #>
int hashhashAt;               //copy of EEPROMindex at hashhashSTOP 
int ReactAt;                  //copy of EEPROMindex at React
int ActEnd;                   //end address of current recording Act in EEPROM
int moves =0;                 //current move
char Ccmnd =0;                //current Character command, default null command
byte epage =1;                //the Act page for the current Act, default page =1 for  Acts 1-8 and page =2 for 01-08

//== HELP TEXT ==============================================
// see http://arduino.cc/en/Reference/PROGMEM

char helpbuffer[80];          //buffer for helptext in PROGMEM
  
const char h_plng[] PROGMEM = "'!' - use page3 Reacts for the next 'r', 'l', '?' or replay"; 
const char h_0[]    PROGMEM = "'0' - use page0 for the next Act"; 
const char h_1[]    PROGMEM = "'1'-'8' the routine or Act to record or replay";
const char h_Q[]    PROGMEM = "'.',[Esc] - quit playing or recording";
const char h_F[]    PROGMEM = "'F' - Forwards one step";
const char h_B[]    PROGMEM = "'B' - Backwards one step";
const char h_R[]    PROGMEM = "'R' - turn Right one step";
const char h_L[]    PROGMEM = "'L' - turn Left one step";
const char h_H[]    PROGMEM = "'H' - Halt with both feet together";
const char h_V[]    PROGMEM = "'V' - Voice, make a beep";
const char h_S[]    PROGMEM = "'S' - Stop/Start - disable/enable servos";
const char h_U[]    PROGMEM = "'U' - read Ultrasonic sensor and react";
const char h_W[]    PROGMEM = "'W' - read Whisker sensors and avoid obstacles";
const char h_G[]    PROGMEM = "'G' - read Beard sensors and avoid holes";
const char h_A[]    PROGMEM = "'A' - beAlive()   until quit";
const char h_X[]    PROGMEM = "'X' - Xplore()   until 'Q' or reset";
const char h_E[]    PROGMEM = "'E' - Experiment - user routine  until 'Q' or reset";
const char h_v[]    PROGMEM = "'v','b','e','o' - choose sound: 'b' bip, 'e' whee, 'o' whoa";
const char h_w[]    PROGMEM = "'w', '1'-'9' - wait 1 - 9 seconds"; 
const char h_su[]   PROGMEM = "'su' - speedup, speed is increased by 1 up to max"; 
const char h_sd[]   PROGMEM = "'sd' - speed down, speed is divided by 2 down to 1"; 
const char h_s[]    PROGMEM = "'s', '1'-'9' set roll and pace speeds to the same"; 
const char h_sr[]   PROGMEM = "'sr', '1'-'9' set roll speed"; 
const char h_sp[]   PROGMEM = "'sp', '1'-'9' set pace speed"; 
const char h_sm[]   PROGMEM = "'sm' - set rollspeedmax, pacespeedmax to current speeds";
const char h_srm[]  PROGMEM = "'srm' - set rollspeedmax to current rollspeed"; 
const char h_spm[]  PROGMEM = "'spm' - set pacespeedmax to current pacespeed"; 
const char h_bn[]   PROGMEM = "'b','#','0'-'9' - robot group  9-0";
const char h_bg[]   PROGMEM = "        'A'-'z' - robot name A-z";
const char h_brun[] PROGMEM = "    'a','0'-'8' - set Autorun act, '0' = no autorun";  
const char h_bdo[]  PROGMEM = "    'a','9'     - set Autorun act to run when there is nothing else to do, ## turns it off";   
const char h_bg0[]  PROGMEM = "    'w0' - set walking off"; 
const char h_bg1[]  PROGMEM = "    'w1' - set walking on"; 
const char h_bp[]   PROGMEM = "    'p','1'-'9' - paceby =2*cmnd - step length";
const char h_bt[]   PROGMEM = "    't','1'-'9' - turnby =2*cmnd - step length in turns";
const char h_br[]   PROGMEM = "    'r','1'-'9' - rollby =2*cmnd - roll amount when stepping";
const char h_bump[] PROGMEM = "    'um+' - manually set SonarAwareAt";
const char h_bumn[] PROGMEM = "    'um-' - manually set SonarTooNear";
const char h_bup[]  PROGMEM = "    'u+','1'-'9' - set SonarAwareAt";
const char h_bun[]  PROGMEM = "    'u-','1'-'9' - set SonarTooNear";
const char h_bus[]  PROGMEM = "    'us' - save current Sonar settings to EEPROM";
const char h_bq[]   PROGMEM = "    'q' - Quiet - turn off status messages";
const char h_bv[]   PROGMEM = "    'v','0','1' - verbose, light level reporting of sensors, 0=off, 1=on";
const char h_badj[] PROGMEM = "    '@' - Adjust Servos"; 
const char h_ta[]   PROGMEM = "'t','a' - turn Away from leading foot"; 
const char h_tm[]   PROGMEM = "    'm' - turn Memory (same way as last turn";
const char h_to[]   PROGMEM = "    'o' - turn Other way from last time";
const char h_tl[]   PROGMEM = "    'l' - turn towards Light";
const char h_td[]   PROGMEM = "    'd' - turn towards Dark";
const char h_tAtl[] PROGMEM = "    '@l' - turn until facing Light";
const char h_tAtd[] PROGMEM = "    '@d' - turn until facing Dark";
const char h_r[]    PROGMEM = "'r','1'-'8' - Record moves into EEPROM";
const char h_c[]    PROGMEM = "'c','1'-'8','1'-'8' -  copy one Act to another Act,  xy,x0y,0xy,0x0y";
const char h_l[]    PROGMEM = "'l','1'-'8' - load Act into EEPROM without playing it"; 
const char h_qh[]   PROGMEM = "'?','h' - print Help - these instructions";
const char h_qx[]   PROGMEM = "    '1'-'8' - print moves in selected Act '1' - '8'";
const char h_qa[]   PROGMEM = "    'a' - print all Acts";
const char h_qq[]   PROGMEM = "    '?' - print all Acts";
const char h_qd[]   PROGMEM = "    'd' - Dump out all acts to the PC"; 
const char h_qb[]   PROGMEM = "    'b' - print Behaviour - behaviour variables in RAM";
const char h_qm[]   PROGMEM = "    'm' - print all Acts, Behaviour";
const char h_qr[]   PROGMEM = "    '!' - print all ReActs";
const char h_mr[]   PROGMEM = "'#','*' - message for all robots, robots silent";
const char h_mn[]   PROGMEM = "    '0-9' - message for robot group 0-9,  robots silent";
const char h_mg[]   PROGMEM = "    'A-z' - message for robot name A-z, in current group, other robots silent";
const char h_ms[]   PROGMEM = "    '#' - (default) stop, obey commands and send messages";    
const char h_ms2[]  PROGMEM = "          useful to talk to an unknown robot or put all in the stopped state"; 
const char h_mAd[]  PROGMEM = "    '<' - do default Act";    
const char h_mAr[]  PROGMEM = "    '>' - resume Act";    
const char h_mq[]   PROGMEM = "    '?' - print all names and groups - delayed depending on name ";    
  
PROGMEM const char * const help_table[] =
  { h_plng, h_0,    h_1,    h_Q,    h_F,    h_B,    h_R,    h_L,    h_H,
    h_V,    h_S,    h_U,    h_W,    h_G,    h_A,    h_X,    h_E,
    h_v,    h_w,    h_su,   h_sd,   h_s,    h_sr,   h_sp,   h_sm,   h_srm,  h_spm,   
    h_bn,   h_bg,   h_brun, h_bdo,  h_bg0,  h_bg1,  h_bp,   h_bt,   h_br,
    h_bump, h_bumn, h_bup,  h_bun,  h_bus,  h_bq,   h_bv,   h_badj,
    h_ta,   h_tm,   h_to,   h_tl,   h_td,   h_tAtl, h_tAtd,
    h_r,    h_c,    h_l,
    h_qh,   h_qx,   h_qa,   h_qq,   h_qd,   h_qb,   h_qm,   h_qr, 
    h_mr,   h_mn,   h_mg,   h_ms,   h_ms2,  h_mAd,  h_mAr,  h_mq    
  };

//== SETUP ==================================================
void setup() 
{ 
  #include "init44.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =4;  // quite fast 
  pacespeedmax =6;  // quite fast  
}
//== LOOP ===================================================
void loop()
{ 
  f_recordC =true;     // record Command, default =true
  
#ifdef BambinoMM 
  redLEDdim();         //indicate ready for command except    
  if(f_recording)  { redLEDon(); }
  if(f_play)       { redLEDoff(); }
#else
  if(f_recording)  { redLEDon(); } else { redLEDoff(); }
  if(!f_reacting)                    // don't change if in a ReAct
    { if(f_play)
        { whiskerLEDsOff();  }
        else
        {  whiskerLEDsDim(); }    //indicate ready for command
      }  
#endif      

  if(!f_play && !f_reacting && !f_BeQuiet){Serial.print(">"); }    //ready prompt
  
  if (f_doDefault)   // true on the first loop and #< only
    { f_doDefault =false;                // don't do it again
      Ccmnd =EEPROM.read(EautoAct);
      if(Ccmnd=='0') { f_runDefault =false;  Ccmnd ='.'; }     // '0' -> no autorun, can't run it so do stop 
    }   
   else
    // if f_hashhashSTOP skip GetCcmnd() and process Ccmnd '.' from ##
    { if(!f_hashhashSTOP) { GetCcmnd(); } }   //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd

DoCommand:
  switch (Ccmnd) 
  { case LF:    f_recordC =false;  break;     //linefeed
    case CR:    f_recordC =false;  break;     //carriage return
    case ' ':   break;     //strip space
    case ',':   break;     //strip comma
    case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
    case '.':   if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }     //terminated string
                f_recording =false;
                if(!f_reacting) 
                  { f_play =false;                                                        // end of Act  
                    if (!f_hashhashSTOP) { EEPROMindex =-1; };   // so there is nothing to continue with #>
                    f_hashhashSTOP =false;
                    if(!f_BeQuiet){Serial.println();}
                    if (f_runDefault) { f_doDefault =true; }
                  } 
                if(f_reacting) { f_reacting =false; EEPROMindex =ReactAt; beardLEDsOff(); }     // end of ReAct
                break;
    case 'A':	  // if recording record 'A' otherwise do beAlive() until 'Q' or reset
                if (!f_recording) { beAlive(); }  break;
    case 'X':	  // if recording record 'X' otherwise do Xplore() until 'Q' or reset
                if (!f_recording) { Xplore(); }  break; 
    case 'E':	  // if recording record 'E' otherwise do Experiment() until 'Q' or reset
                if (!f_recording) { Experiment(); }  break; 
    case '!':   epage =3;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '0':   epage =2;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '1':   // fall through
    case '2':   // fall through
    case '3':   // fall through
    case '4':   // fall through
    case '5':   // fall through
    case '6':   // fall through
    case '7':   // fall through
    case '8':   if (!f_recording) { f_play =true; Act =Ccmnd;  setEEPROMindexes(Act); }  break;
    case 'F':   forward();      break;        
    case 'B':   backward();     break;        
    case 'R':   turnRight();    break;        
    case 'L':   turnLeft();     break;        
    case 'H':   halt();         break; 
    case 'V':   whee();         break;   //Spkr and LED on different pins           
    case 'S':   f_recordC =false;
                toggle_f_walking(); 
                if(!f_BeQuiet)
                  {if (f_walking) {Serial.println(" walking");} else {Serial.println(" stopped");}}
                break;    
    case 'U':   USreact();      break; 
    case 'W':   WhiskerReact(); break; 
    case 'G':   BeardReact();   break;
    case 'v':   if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd) 
                { case 'b':  bip();   break;
                  case 'e':  whee();  break;
                  case 'o':  whoa();  break;
                  default:   bip();   Ccmnd ='b';  break;
                }    // end switch
                break;                
    case 'w':   if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded
                GetCcmnd();                            //get parameter
                makeCcmndDigit();                      //ensure '1'-'9'
                waitCcmndSecs();
                break;                                 // if recording Ccmnd will be recorded
    case 's':   if (f_recording) { RecordCcmnd(); }    //so 's' is recorded
                GetCcmnd();                            //get parameter
                if (Ccmnd=='u') { speedup();  break; }     // and if recording record 'u' 
                if (Ccmnd=='d') { speeddown();  break; }   // and if recording record 'd' 
                if (Ccmnd=='m') { rollspeedmax =rollspeed; pacespeedmax =pacespeed; break; }
                if (Ccmnd=='r') 
                { if (f_recording) { RecordCcmnd(); }  //so 'r' is recorded
                  GetCcmnd();                          //get parameter
                  if (Ccmnd=='m') { rollspeedmax =rollspeed; break; }
                  //else                 
                  makeCcmndDigit();                    //ensure '1'-'9'    
                  rollspeed =Ccmnd -'0';
                  break;                               // and if recording record digit
                }
                if (Ccmnd=='p') 
                { if (f_recording) { RecordCcmnd(); }  //so 'p' is recorded
                  GetCcmnd();                          //get parameter
                  if (Ccmnd=='m') { pacespeedmax =pacespeed; break; }
                  //else                 
                  makeCcmndDigit();                    //ensure '1'-'9'    
                  pacespeed =Ccmnd -'0';
                  break;                               // and if recording record digit 
                }
                //else                 
                makeCcmndDigit();                      //ensure '1'-'9'    
                rollspeed =max(1,Ccmnd -'0'); 
                pacespeed =max(1,Ccmnd -'0');       	
                break;                                 // and if recording record digit 
    case 't':   if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { case 'a':  turnAway();    break;
                  case 'm':  turnMemory();  break;
                  case 'o':  turnOther();   break;
                  case 'l': turnLight();   break;
                  case 'd': turnDark();    break;
                  case '@': if (f_recording) { RecordCcmnd(); }    //so '@' is recorded
                            GetCcmnd();                            //get parameter
                            switch (Ccmnd)
                            { case 'l':  turnAtLight();  break;
                              case 'd':  turnAtDark();   break;
                              default:   badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                                         Ccmnd ='?';     //so if recording then '?' is recorded
                                         break;          //don't do anything 
                            }    // end switch
                  default:  turnMemory();   Ccmnd ='m';  break;
                }     // end switch case t
                break;                               
    case 'b':   if (f_recording) { RecordCcmnd(); }    //so 'b' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { case '#': f_recordC =false;
                            GetCcmnd();                            //get parameter
                            if( f_BeQuiet==1 ) { break; }     // don't change these if being quiet
                            if (Ccmnd>='0' && Ccmnd<='9')
                              { rgroup =Ccmnd;  EEPROM.write(Ergroup,rgroup);  break; }
                            if (Ccmnd>='A' && Ccmnd<='z') 
                              { rname =Ccmnd;  EEPROM.write(Ername,rname);  break; }
                            badkey();  break;
                  case 'a': if (f_recording) { RecordCcmnd(); }    //so 'a' is recorded
                            GetCcmnd();                            //get parameter
                            if(Ccmnd<'0' || Ccmnd>'9') { Ccmnd ='0'; break; }
                            if (Ccmnd=='9') {f_runDefault =true; break;}             // ## sets f_runDefault false
                            EEPROM.write(EautoAct,Ccmnd);
                            break;
                  case 'w': if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded 
                            GetCcmnd();                            //get parameter
                            if (Ccmnd=='0') { disable_walking();  break; } // and if recording record '0' 
                            if (Ccmnd=='1') { enable_walking();  break; }   // and if recording record '1' 
                            Ccmnd ='?';   // bad char, no action and if recording record '?' 
                            break;                                 // if recording Ccmnd will be recorded
                  case 'p': if (f_recording) { RecordCcmnd(); }    //so 'p' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            paceby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;
                  case 't': if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            turnby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;
                  case 'r': if (f_recording) { RecordCcmnd(); }    //so 'r' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            rollby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;         
                  case 'u': if (f_recording) { RecordCcmnd(); }    //so 'u' is recorded
                            GetCcmnd();                             //get parameter
                            switch (Ccmnd)
                            { case 'm': // don't record 'm'
                                        do
                                        { Serial.println("press [+] for AwareAt, [-] for TooNear, [.] or [Esc] to Quit");
                                          GetCcmnd();                            //get parameter
                                          objectRange =dist_HC_SR04(); if(f_cm) {objectRange /=5;} else {objectRange /=2;}
                                          if (Ccmnd=='+') 
                                            { SonarAwareAt =objectRange; 
                                              if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                              Ccmnd ='0' +SonarAwareAt; if (f_recording) { RecordCcmnd(); }    //so 'SonarAwareAt' is recorded
                                              }
                                          if (Ccmnd=='-')
                                            { SonarTooNear =objectRange;
                                              if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                              Ccmnd ='0' +SonarTooNear; if (f_recording) { RecordCcmnd(); }    //so 'SonarTooNear' is recorded
                                              }
                                        } while (Ccmnd!='.' &&  Ccmnd!='Q' && Ccmnd!=Esc);
                                        break;
                              case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarAwareAt =Ccmnd -'0';
                                        break;
                              case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarTooNear =Ccmnd -'0';
                                        break;
                              case 's': EEPROM.write(ESonarAwareAt,SonarAwareAt);          
                                        EEPROM.write(ESonarTooNear,SonarTooNear);          
                                        break;
                            }     // end switch
                            break;
                  case 'q': f_BeQuiet =1;  break;           
  b_verbose:      case 'v': if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded          
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='1') { f_bv =1; }
                            else 
                            { Ccmnd ='0'; f_bv =0; }               //default '0'   
                            break;
                  case '@': if( f_BeQuiet==1 ) { break; }     // don't adjust if being quiet
                            AdjustServos();  break;           
                  default:  badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                            Ccmnd ='?';     //so if recording then '?' is recorded
                            break;          //don't do anything
                }     // end switch
                break;                               
    case 'r':   f_recordC =false;
                redLEDon();
                if (!f_recording) 
                  { getAct();  if(Ccmnd=='.') { break; }  
                  setEEPROMindexes(Act);  f_recording =true; 
                  }
                break;
    case 'c':   f_recordC =false;  if (!f_recording) { copyAct(); }  break;	
    case 'l':   f_recordC =false;  if (!f_recording) { loadAct(); }  break;	
    case '?':   f_recordC =false;
                if (!f_recording) 
                  { GetCcmnd();                           //get parameter
                    if( f_BeQuiet==1 ) { break; }         // don't print if being quiet
                    switch (Ccmnd)
                    { case 'h':  printHelp();  break;     // print instructions
                      case '1':  // fall through
                      case '2':  // fall through
                      case '3':  // fall through
                      case '4':  // fall through
                      case '5':  // fall through
                      case '6':  // fall through
                      case '7':  // fall through
                      case '8':  Act =Ccmnd;  printAct();  break;
                      case 'p':  PrintAllActs();  break;
                      case '?':  PrintAllActs();  break;
                      case 'd':  DumpActs();  break;
                      case 'b':  printBehaviour();  break;
                      case 'm':  printMind();  break;      // - all Acts, Behaviour
                      case '!':  PrintReActs(); break;
                      default:   badkey();  if(!!f_BeQuiet){Serial.println("?");}  break;
                    }    // end switch               
                  }    // end if 
                break;
    case '#':   f_recordC =false;
                f_MsgForMe =1;
                f_hashcmnd =true;           // don't get a Play command here
                GetCcmnd();                 //get parameter
                f_hashcmnd =false;
                switch (Ccmnd)
                { case '*': f_MsgForMe =1;  f_BeQuiet =1;  hashgroup =rgroup;  break;  //everybody listen
                  case '#': f_MsgForMe =1;  f_BeQuiet =0;                  //everybody stop
                            disable_walking();  Ccmnd ='.';  hashhashAt =EEPROMindex;
                            f_hashhashSTOP =true;  // so '.' gets processed
                            f_runDefault =false;
                            break;
                  case '<': f_doDefault =true;  f_hashhashSTOP =false;  enable_walking();  break;
                  case '>': if (hashhashAt!=-1)                //something to continue
                            { f_play =true;  enable_walking(); EEPROMindex =hashhashAt; }
                            f_hashhashSTOP =false;    
                            break;
                  case '?': delay((rgroup -'0')*50+(rname-'A')*10);
                            Serial.print(" "); Serial.print(rgroup); Serial.print(rname); 
                            break;
                  case '.': f_MsgForMe =0;  f_BeQuiet =1;  hashgroup =' ';     //end #session
                            break;
                  default:  if (Ccmnd>='0' && Ccmnd<='9' ) { 
                              hashgroup =Ccmnd; 
                              if (hashgroup==rgroup) 
                                { f_MsgForMe =1;  f_BeQuiet =1;  }    //my group
                               else
                                { f_MsgForMe =0;  f_BeQuiet =1;  }    //not my group
                              break;
                              }
                            if (Ccmnd==rname && hashgroup==rgroup)  { f_MsgForMe =1;  f_BeQuiet =0;  break; }    //me
                            f_BeQuiet =1;      // any other key             //not for me 
                            break;
                }
                break;              
                
    default:    f_recordC =false;    //don't record bad key		
                badkey();  if(!f_BeQuiet){ Serial.println("bad key"); }  break;
  }    // end switch
	
Record:	
  if (f_recording  && f_recordC) { RecordCcmnd(); } 
  
}    
//== FUNCTIONS ==============================================

void GetCcmnd()	       //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
{ Ccmnd =0;
  while(Ccmnd==0)
  { if(f_recording)  
    { while(!Serial.available()) {}       //wait for Serial
      Ccmnd =Serial.read();  bip(); 
    } 
    else         //if(!f_recording)
    { if(Serial.available()) 
      { Ccmnd =Serial.read();            //over-ride if Serial, null ignored
        if(!f_MsgForMe && Ccmnd!='#') { Ccmnd =0; }     //discard messages for others
        else { bip(); }
      }  
      else 
      { if(!f_hashcmnd)         //don't get a Play character if in the middle of a hash command   
        { if(f_play || f_reacting)
          { Ccmnd =PlayAct();             //get Act Ccmnd 
          }
        }  
      }
    }  //end if(!f_recording)                                               
  }  //end while Ccmnd==0
  if( !f_BeQuiet ){ Serial.print(Ccmnd); }
}   
//============================  
void RecordCcmnd()	
{ EEPROM.write(EEPROMindex,Ccmnd);
  EEPROMindex++;
  if (EEPROMindex>=ActEnd) { f_recording =false;  EEPROM.write(ActEnd,'.');  badkey(); }     //terminated string 
}  
//============================  

void Experiment()       //until Quit
{ while(true)
  { 
    //put your own routine here e.g.
    rollspeed =4;
    pacespeed =4;
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    whee(); 
    turnRight();
    turnRight();
    turnRight();    
    whee(); 
    rollspeed =2;
    pacespeed =2;
    turnLeft();
    turnLeft();
    turnLeft();
    whee(); 
    rollspeed =1;
    pacespeed =1;
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    whee();    
    whee();    
    whee();    
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------
void Xplore()         //Wander() until Quit
{ while(true)
  { Wander();
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------  
void makeCcmndDigit()      //ensure 1-9
{ if (Ccmnd<'1' || Ccmnd>'9') { Ccmnd ='1'; } //default
}
//----------------------------  
void waitCcmndSecs()   //read delay time from Act and wait
{ if (Ccmnd<'1' || Ccmnd>'9') { Ccmnd ='1'; } //default
  byte cmnd =Ccmnd -'0';                          // ASCII to numeric
  delay(cmnd *1000);
}  
//----------------------------  
void printMind()
{ PrintAllActs();
  printBehaviour();
}
//----------------------------
void PrintReActs()
{ Serial.println();  Serial.print("W & G, 1 indicates object or hole sensed LR");
  for(Act ='1'; Act<='8'; Act++) { epage =3; printReAct(); }  Serial.println();  
}
//----------------------------
void printReAct()
{ Serial.println();
  Serial.print("ReAct");  Serial.print(Act);  Serial.print(" ");
  switch (Act)
  { case '1':  Serial.print("U<=");  Serial.print(SonarAwareAt);  Serial.print(" ");  break;
    case '2':  Serial.print("U<=");  Serial.print(SonarTooNear);  Serial.print(" ");  break;
    case '3':  Serial.print("W11");  Serial.print("  ");  break;
    case '4':  Serial.print("W10");  Serial.print("  ");  break;
    case '5':  Serial.print("W01");  Serial.print("  ");  break;
    case '6':  Serial.print("G11");  Serial.print("  ");  break;
    case '7':  Serial.print("G10");  Serial.print("  ");  break;
    case '8':  Serial.print("G01");  Serial.print("  ");  break;
  } // end switch
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);    //there are moves which take more than 1 character  
  Serial.print(" bytes"); //so the space taken in EEPROM is really bytes
  moves =0;	
}
//----------------------------  
void printBehaviour()
{ Serial.println();                
  Serial.println(thisprog); // so we know which program is running
  Serial.print("robot ");      Serial.print(rgroup);   Serial.print(rname); 
                               Serial.print(" MsgForMe=");  Serial.print(f_MsgForMe); 
                               Serial.print(" BeQuiet=");   Serial.println(f_BeQuiet);  
  Serial.print(f_walking);     Serial.println(" walking");
  Serial.print(char(EEPROM.read(EautoAct))); Serial.print(" autoAct,  "); Serial.print("rerun "); if (f_runDefault==1) { Serial.print("On"); } else { Serial.print("Off"); } Serial.println("  (default Off)"); 
  Serial.print(rollspeed);     Serial.println(" rollspeed");
  Serial.print(pacespeed);     Serial.println(" pacespeed");
  Serial.print(rollspeedmax);  Serial.println(" rollspeedmax");
  Serial.print(pacespeedmax);  Serial.println(" pacespeedmax");  
  Serial.print(rollby);        Serial.print('(');        Serial.print(rollby/2);        Serial.print(')');        Serial.println(" rollby");
  Serial.print(paceby);        Serial.print('(');        Serial.print(paceby/2);        Serial.print(')');        Serial.println(" paceby");
  Serial.print(turnby);        Serial.print('(');        Serial.print(turnby/2);        Serial.print(')');        Serial.println(" turnby");
  Serial.print(rollC);         Serial.println(" rollC");    
  Serial.print(paceC);         Serial.println(" paceC");   
  Serial.print(SonarAwareAt);  Serial.println(" SonarAwareAt");   
  Serial.print(SonarTooNear);  Serial.println(" SonarTooNear");   
  Serial.print(f_bv);          Serial.println(" verbose");
}  
//----------------------------  
void DumpActs()        //Dump Acts, Acts0, ReActs. Print without names and length
{ for(Act ='1'; Act<='8'; Act++)
  { setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =2;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =3;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  Serial.println();
}		
//----------------------------  
void loadAct()
{ redLEDon();  
  getAct();                         // Ccmnd ->'.' - Quit,
                                    // '0', '!'    - sets Act0 or ReAct page then gets Act
                                    // '1' - '8'   - sets Act page'1' - '8',  - 
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    while(true) 
    { while(!Serial.available()) {}  //wait for Serial
      Ccmnd =Serial.read();  Serial.print(Ccmnd);
      if(Ccmnd=='.' || Ccmnd==Esc) { break; }
      EEPROM.write(EEPROMindex,Ccmnd);  EEPROMindex++;
      if (EEPROMindex>=ActEnd) { badkey();  break; }     //ActEnd =EEPROMindex +31;   
    }
    EEPROM.write(EEPROMindex,'.');                        //terminated string 
  }
}
//----------------------------  
void copyAct()
{ getAct();
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    int fromEEPROMindex =EEPROMindex;  
    int fromActEnd =ActEnd;
    getAct();
    if (Ccmnd!='.')
    { setEEPROMindexes(Act);
      for(byte i =0; i<=fromActEnd -fromEEPROMindex; i++)
      { EEPROM.write(EEPROMindex +i,EEPROM.read(fromEEPROMindex +i)); }
    } 
  }	
}
//----------------------------
char PlayAct() 
{ char _Ccmnd =EEPROM.read(EEPROMindex++);
  if(_Ccmnd==255 || _Ccmnd==-1) _Ccmnd ='.';     //uninitialised EEPROM value, don't play it, change to '.'
  return _Ccmnd;
}  
//----------------------------
void PrintAllActs()
{ for(Act ='1'; Act<='8'; Act++) { epage =1; printAct(); }  Serial.println(); 
  for(Act ='1'; Act<='8'; Act++) { epage =2; printAct(); }  Serial.println(); 
}
//----------------------------
void printAct()
{ Serial.println();
  Serial.print("Act");
  if(epage==2) { Serial.print('0'); }    //for logical page0 Acts
  Serial.print(Act);
  Serial.print(" ");
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);
  Serial.print(" bytes");
  moves =0;	
}
//----------------------------  
void getAct()      //called when recording, loading, printing and copying Acts, Ccmnd ->'.', '1' - '8'
{ Ccmnd =0;
  do
  { if(Serial.available()) 
    { Ccmnd =Serial.read();   Serial.print(Ccmnd);
      switch (Ccmnd) 
      { case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  Serial.print(">");  break;     //quit, print ready prompt
        case '!':  epage =3;  Ccmnd =0;  break;  //do it again to read Act
        case '0':  epage =2;  Ccmnd =0;  break;  //do it again to read Act
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':  Act =Ccmnd;  break;
        default: badkey(); Serial.print(" enter Act or '.'"); Ccmnd =0;	
      }  //end case
    } //end if Serial
  } while(Ccmnd==0);
  Serial.print(Ccmnd);
}    
//----------------------------
void setEEPROMindexes(char Act)     // set EEPROM pointers
{ actIndex =Act -'1';               // ASCII to numeric,  Act 1 == actIndex 0
  EEPROMindex =(256 *epage) +(actIndex*32);  
  ActEnd =EEPROMindex +31;          // used in record, load, copy
  epage =1;            //set to default page, Acts 1 - 8 
}
//----------------------------
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF  
  whoa();  
}  
//----------------------------
void printHelp()
{ Serial.println();
  for (int i = 0; i < (sizeof(help_table)/sizeof(int)); i++) 
  //sizeof gives bytes so divide by sizeof(int) to get number of elements
  {
    // see http://arduino.cc/en/Reference/PROGMEM
    strcpy_P(helpbuffer, (char*)pgm_read_word(&(help_table[i]))); // Necessary casts and dereferencing
    Serial.println( helpbuffer );
  }  
}
//----------------------------
void USreact()     // "U" read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
{ objectRange =dist_HC_SR04();
  if( !f_BeQuiet ) { Serial.println(); }
  if( !f_BeQuiet ) { Serial.print("Ultrasonic distance ="); }
  if( !f_BeQuiet ) { Serial.print(objectRange); }
  if( !f_BeQuiet ) { if(f_cm) {Serial.println("cm");} else {Serial.println("inch");} }
  reactUObjects();
}
//----------------------------
void WhiskerReact()     // "W" read Whisker sensors and avoid obstacles (if any) using avoidObstacles() behaviour rules
{ readWhiskers(); 
  queryObstaclesLR();
  reactObstaclesLR();
}
//----------------------------
void BeardReact()     // "G" Ground - read Beard sensors and avoid holes (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdgeLR();
  reactHolesLR(); 
}
//----------------------------
void beAlive()         // the loop() from B_PM22_BeAlive
{ while(true)
  { switch (random(9))
    { case 0: WanderSteps(random(10)); break;    //uses all sensors
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
    } // end switch
    if( Quit() ) { break; }
  }	// end while
}
//---------------------------
boolean Quit()          //check for Quit
{ if( Serial.available() ) 
  { Ccmnd =Serial.peek(); 
    if( Ccmnd=='.' || Ccmnd==Esc || Ccmnd=='#')
      { return true; }
     else
      { return false; }
  }  
}
//----------------------------
void bip()            // make a bip sound
{ beep(voicePin,1500,50); }
//----------------------------
#ifdef BambinoMM
void redLEDdim()
{ analogWrite(redLEDpin_,254); }  //redLED, LOW=on so 255 is off 
//----------------------------
void redLEDon()
{ digitalWrite(redLEDpin_,LOW); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin_,HIGH); }  
//---------------------------- 
//BambinoMM does not have Whisker sensors
//---------------------------- 
#else
void redLEDon()
{ digitalWrite(redLEDpin,HIGH); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin,LOW); }  
//----------------------------
void whiskerLEDsOff()
{ digitalWrite(RwhiskerLEDpin_, HIGH);     // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);     // whisker LED OFF
  }  
//----------------------------  
void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
  }
#endif
//----------------------------  
void beardLEDsOff()
{ digitalWrite(RbeardLEDpin_, HIGH);       // beard LED Off
	digitalWrite(LbeardLEDpin_, HIGH);       // beard LED Off
  }
//===========================================================
