//BA45_Behaviour.ino

char thisrobot[] ="Bambino ";
char thisprog[]  ="BA45_Behaviour"; 

// Sketch uses 24990 bytes (77%) of program storage space. Maximum is 32256 bytes.
// Global variables use 1012 bytes (49%) of dynamic memory, leaving 1036 bytes for local variables. Maximum is 2048 bytes.

//set Serial terminal to 9600 baud
//== #define ================================================
//#define BambinoPM         //ProMini 328,5v
//#define BambinoMM         //Uno with no Whisker sensors
#define BambinoM2         //Uno 

/*
== Notes ====================================================
  
Introducing
  Previously there was just one behaviour with various behaviour parametes.
  Now there are eight behaviours taking up all of the first 255 bytes of EEPROM 
  and just as with Acts each behaviour takes up 32 bytes. 
  b0     - reload cuurent behaviour to RAM BeHaves  
  b,1-8  - set behaviour slot and copy contents to current behaviour in RAM
  bs,1-8 - save current behaviour to a slot
  bl,1-8 - load a behaviour slot, with
           <decimal behavior-number>,<parameter>,....    behavior-number =0 ends
           parameter is <char>,#<decimal byte value>, etc. depends on Terminal software used.
           eg bl4 30,85.      load slot 4 position 30 with 85 and quit loading
           eg bl5 30,85,20,2. load slot 5 position 30 with 85 and position 20 with 2 and quit loading
  ?B - print all Behaviour slots as a table

  Toggling f_walking in setup not needed now there is more than one behaviour, it was 
  needed to stop walking when testing sensors but since that needs a terminal, a terminal
  can be used to turn off f_walking. 
============================================================
Note - The Arduino Serial Monitor can't send [Esc]. 
       If another Terminal is used which can send [Esc] then at GetCcmnd() this will be printed,
	     which may print as a symbol or the Terminal may change it to "."
     
Bugs -	   
============================================================
improvements? - 
- void GetCcmnd()	  could use   case 0:   - why?

To do next - 

Spare slot addresses   
26 - 23 could be used as well
12 bG<0-9> - automatic sense Ground,     0=off, 1-9 triggering every x times through the cmnd loop
11 bW<0-9> - automatic sense Whiskers,   0=off, 1-9 triggering every x times through the cmnd loop
10 bU<0-9> - automatic sense Ultrasonic, 0=off, 1-9 triggering every x times through the cmnd loop
9
8  LeftRightLightTurnThresholdScale - 1-9 in units of 1%
7  LeftRightLightTurnThreshold      - 0-9 in units of Scale
6  holeThresholdScale     - 1-9 in units of 1%
5  holeThresholdRight     =10; % - 0-9  units of 5%  // difference between RbeardLight and RbeardDark for a hole
4  holeThresholdLeft      =10; % - 0-9  units of Scale // difference between LbeardLight and LbeardDark for a hole 
3  obstacleThresholdScale - 0-9  units of 1%  
2  obstacleThresholdRight  =20; % - 0-9  units of 5%  // difference between RwhiskerLight and RwhiskerDark for an obstacle
1  obstacleThresholdLeft   =20; % - 0-9  units of 5%  
--------------------------------------  
Cr+,Cr-(1-9) - alter rollC in current mind in RAM - good for walking across slopes
Cp+,Cp-(1-9) - alter paceC in current mind in RAM

Superceded by ReActs
16 bytes for behaves => 8 in 128 bytes
sr,sp,srm,spm,mr,mp,mt,u-,u+,(spare 7)
two bytes each => 16 bytes, => 8 in 128 bytes
bG-<{0},1-8> - Act to be done on hole-tests left & right true 
bGR<{0},1-8> - Act to be done on hole-test right true 
bGL<{0},1-8> - Act to be done on hole-test left  true 
bW-<{0},1-8> - Act to be done on Whisker-tests left & right true 
bWR<{0},1-8> - Act to be done on Whisker-test right true 
bWL<{0},1-8> - Act to be done on Whisker-test left  true
bU+<{0},1-8> - Act to be done on Ultrasonic distance <= aware at
bU-<{0},1-8> - Act to be done on Ultrasonic distance <= too near
=============================================================
Primary commands used
012345678 ?#. ABEFGHLRSUVWX bclrstvw
   
COMMANDS
======== 
'.',[Esc] - quit current operation - recording, playing, input
'S' - Stop/Start
'!' - use page3 Reacts for the next record, load, replay, or print
'0' - use page0 for the next Act
'1'-'8' the routine or Act to record or replay
'F' - Forwards one step
'B' - Backwards one step
'R' - turn Right one step
'L' - turn Left one step
'H' - Halt with both feet together
'V' - Voice, make a beep, whee
'U' - read Ultrasonic sensor and react to objectRange using reactUObjects() and ReActs in EEPROM
'W' - read Whisker sensors and avoid holes (if any) using reactObstacles() and ReActs in EEPROM
'G' - read Beard sensors and avoid obstacles (if any) using reactHoles() and ReActs in EEPROM
'A' - beAlive()  until '.' or Esc or reset      // the loop() from B_PM22_BeAlive
'X' - Xplore()   until '.' or Esc or reset      // Wander()
'E' - Experiment - user routine  until '.' or Esc or reset
'v', 'b','e','o' choose sound: 'b' bip, 'e' whee, 'o' whoa
'w', '1'-'9' - wait 1 - 9 seconds 
'su' - speedup, speed is increased by 1 up to max
'sd' - speed down, speed is divided by 2 down to 1
's','1'-'9' set speed, both roll and pace speeds set to the same
'sr','1'-'9' set roll speed                              units of servo degrees change per frame
'sp','1'-'9' set pace speed                              units of servo degrees change per frame
'sm' - set rollspeedmax to current rollspeed, set pacespeedmax to current pacespeed
'srm' - set rollspeedmax to current rollspeed
'spm' - set pacespeedmax to current pacespeed
'b','0' - reload cuurent behaviour to RAM BeHaves
    '1'-'8' - set current behaviour and copy to RAM BeHaves
	's0' - save RAM BeHaves to current EEPROM Behaviour
    's','1'-'8' - save RAM BeHaves to EEPROM Behaviour
    'l','1'-'8' - load EEPROM behaviour 1-8, at prompt enter BeHave number and BeHave data
           eg bl4 prompt 30,85.      load slot 4 position 30 with 85 and quit loading
           eg bl5 prompt 30,85,20,2. load slot 5 position 30 with 85 and position 20 with 2 and quit loading
        '9' - load entry in all behaviours  
           eg bl9 prompt 22,B.      load group 'B' to all Behaviours and quit loading
           eg bl9 prompt 22,B,21,2. load group 'B', name 2 to all Behaviours and quit loading
    '#','A'-'z' - robot name  A-z
        '0'-'9' - robot group 0-9
    'a','0'-'8' - set Autorun act '1' - '8', '0' no Autorun act   
    'a','9'     - set Autorun act to run when there is nothing else to do, ## turns it off 
                  not remembered through reset/power-off - default off  
    'q' - Quiet - turn off status messages
    'w0' - turn off walking 
    'w1' - turn on walking
    'p','1'-'9' - set paceby - step length                units of 2 servo degrees
    't','1'-'9' - set turnby - step length in turns       units of 2 servo degrees
    'r','1'-'9' - set rollby - roll amount when stepping  units of 2 servo degrees
    'um+' - set manually SonarAwareAt, units 50mm
    'um-' - set manually SonarTooNear, units 50mm
    'u+','0'-'9' - set SonarAwareAt, units 50mm    
    'u-','0'-'9' - set SonarTooNear, units 50mm    
    'uc0' - Sonar distance in inches   
    'uc1' - Sonar distance in centimetres      
    'us' - save current Sonar settings to EEPROM
    'v','0','1' - verbose, light level reporting of sensors, 0=off, 1=on
    '@' - Adjust Servos 
't','a' - turn Away from leading foot, will be used in avoid Acts
    'm' - turn Memory (same way as last turn, will be used in avoid Acts
    'o' - turn Other way from last time, will be used in avoid Acts 
    'l' - turn to light
    'd' - turn to dark
    '@l' - turn until @ light
    '@d' - turn until @ dark    
'r','1'-'8' - Record moves into selected Act in EEPROM
'c','1'-'8','1'-'8' -  copy one Act to another Act, xy,x0y,0xy,0x0y
'l','1'-'8' - load Act, like 'r' but records commands into EEPROM without playing them
'?','h' - print Help instructions
    '1'-'8' - Print moves in selected Act '1' - '8'
    'p' - print current Page
    '?' - print current Page
    'd' - Dump out all acts to the PC, same as 'p' but without the Act and moves numbers.
    'b' - print Behaviour - behaviour variables in RAM
    'B' - print all Behaviours in EEPROM
    'm' - print Mind - all Acts, Behaviour
    '!' - print ReActs
'#','*' - message for all robots, robots silent
    '0-9' - message for robot group 0-9, robots silent
    'A-z' - message for robot name A-z in current group, other robots silent
    '#' - (default) robots stop, obey all commands and send messages     
          useful to talk to an unknown robot or put all in the stopped state 
    '<' - do Default Act   
    '>' - resume Act   
    '?' - all robots reply with their name and group - delayed according to name
   
The whiskerLEDs glowing dim indicate Bambino is ready for a command.
If other keys are pressed, including lower case keys, then the Whisker LEDs will flash badkey().
(MM - mouth glows dim while ready and Beard LEDs used for badkey().) 

RESPONSES
sense44.ino - readLight()  // read ambient Light levels   0 - 1023
  [a<LeyeLight>|<ReyeLight>]
sense44.ino - readWhiskers()
  [W<LwhiskerLight>,<LwhiskerDark>,<LwhiskerLight -LwhiskerDark>|<RwhiskerLight>,<RwhiskerDark>,<RwhiskerLight -RwhiskerDark>]
sense44.ino - readBeard()
  [B<LbeardLight>,<LbeardDark>,<LbeardLight -LbeardDark>|<RbeardLight>,<RbeardDark>,<RbeardLight -RbeardDark>]
sense44.ino - showObstacles()
  [o%<f_obstacleLeft><f_obstacleRight>]
sense44.ino - showHoles()
  [h%<f_holeLeft><f_holeRight>]
=============================================================
ReActs
======
Reactions stored in EEPROM page 3 as 8 slots of 32 bytes, Reactions can have the same commands as Acts and ended with ".".
When a sensor test is made in an Act and something is detected then the relevant Reaction is played then the Act continues.
  Reaction-1 - Reaction if Ultrasonic distance is less than 'aware at' value  
                 hence objects farther away than 'aware at' value are ignored
  Reaction-2 - Reaction if Ultrasonic distance is less than 'too close' value
                 i.e. objects are too close
  Reaction-3 - Reaction if both Left and Right Whiskers sense an object
  Reaction-4 - Reaction if Left Whisker senses an object
  Reaction-5 - Reaction if Right Whisker senses an object 
  Reaction-6 - Reaction if both Left and Right Beard sensors detect a hole
  Reaction-7 - Reaction if Left Beard sensor detects a hole 
  Reaction-8 - Reaction if Right Beard sensor detects a hole 
=============================================================
BEHAVES - power-on defaults set from current default Behaviour in EEPROM
=======
sense behaves
  SonarAwareAt  is EEPROM value  
  SonarTooNear  is EEPROM value
speed behaves  - set in void setup()  
  rollspeed     is EEPROM value            
  pacespeed     is EEPROM value
  rollspeedmax  is EEPROM value
  pacespeedmax  is EEPROM value
move behaves   - set in void setup()  init42.h 
  rollby   is EEPROM value,    set by B_PM04_Servos or AdjustServos()
  paceby   is EEPROM value,    set by B_PM04_Servos or AdjustServos()
  turnby   is paceby
varsCons44.h
  rollC =90;    default, AdjustServos39.ino, roll centre = 90 degrees (ie 1500us) => upright
  paceC =90;    default, AdjustServos39.ino, pace centre = 90 degrees (ie 1500us) => feet together  
  Servoframe =20; never changed, Servo frame rate millisecs
//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pins20.h     - pin usage
//     varsCons44.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h       - the Arduino EEPROM library
//     avr/pgmspace.h - to allow the printHelp() text to be in ProgramSpace
//== VARIABLES and CONSTANTS 
//== HELP TEXT 
//     PROGMEM
//== SETUP
//== LOOP
//== USER ROUTINES
//   Experiment()       //until Quit
//   Xplore()          //Wander() until Quit
//   beAlive()         // the loop() from BA22_BeAlive
//== FUNCTIONS
//   GetCcmnd()	       //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
//   RecordCcmnd()	
//   makeCcmndDigit()  ensure 1-9
//   waitCcmndSecs()   //read delay time from Act and wait
//   printMind()
//   PrintReActs()
//   printReAct()
//   loadBehaviour()
//   get1char()
//   getnum()
//   printBehaviour()
//   PrintBehaviours()
//   DumpActs()        //Dump Acts, Print without names and length
//   loadAct()
//   copyAct()
//   Ccmnd <- PlayAct() 
//   PrintAllActs()
//   printAct()
//   Act <- getAct()  //called when recording, loading, printing and copying Acts 
//   setEEPROMindexes(char Act)      //  changes EEPROMindex, ActEnd
//   badkey()         // indicate invalid key
//   USreact()        // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
//   WhiskerReact()   // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
//   BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
//   redLEDdim() - BambinoMM only
//   redLEDon()
//   redLEDoff() 
//   bip()            // make a bip sound
//   whiskerLEDsDim() - not BambinoMM
//   whiskerLEDsOff() - not BambinoMM
//TAB FILES
//   acts40.ino
//     ACTS - WanderSteps(steps), Wander()
//            RightStep(steps), LeftStep(steps)
//            rock(repeats), shuffle(repeats)
//     SOUND -  whee(), whoa(), beep(byte beeppin,int beepfreqHz,long beeptime)  
//   adjustServos45.ino
//     AdjustServos(), printEEPROMvalues(), printAdjustInstructions() 
//     PROGMEM adjust_instructions
//   behaviour45.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//                           disable_walking(), enable_walking(), toggle_f_walking()
//     BEHAVIOUR RULES - reactUObjects(), reactObstaclesLR(), reactHolesLR()
//                     - avoidUObjects(), avoidObstaclesLR(), avoidHolesLR()
//   help45.ino
//     printHelp() - Help text
//   mind45.ino
//     MIND - getMind(), saveMind(),
//            initialiseEEPROM(), initBehaviour8(), initialiseEEPROMbehaviours1to7()  
//   movement34.ino
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(), 
//                halt(), 
//                rollCentre(), rollLeft(), rollRight(),
//                paceCentre(), paceLeftForward(),  paceRightForward(), 
//                              paceLeftBackward(), paceRightBackward(), 
//                              turnLeftForward(),  turnRightForward(), 
//                              turnLeftBackward(), turnRightBackward(), 
//                rollmove(), pacemove()
//   sense44.ino
//     SENSE - readUSdistance(), showUSdistance(), dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels(), showObstacles() 
//           - readBeard(), QueryEdgeLR(), showBeardLightLevels(), void showHoles()
//           - readLight()
//   servos40.ino
//     SERVOS - ServosInitC()
//              ServoPulse(byte Spin, int Spulse)
//   skills41.ino
//     SKILLS - turnLight(), turnDark(), turnAtLight(), turnAtDark()
//     META MOVEMENT - turnAway(), turnMemory(), turnOther()  
//===========================================================
*/

/*== NEW ====================================================
char Behaviour;               //read from EEPROM in init, current behaviour 1-8 in epage 0
byte BehaviourEEbase =224;    //base address of current behaviour 0,32,64,96,128,160,192,(224)
byte slotbase;                //working slotbase so current BehaviourEEbase is not changed
byte behaveNum;               //BeHave address in Behaviour slot 
byte behaveValue;
String numString ="";         //temporary string in loadBehaviour(), getnum()
void PrintBehaviours()
help.ino - now contains all the help text as Serial.print(F statements
     no longer needs #include <avr/pgmspace.h>     // for PROGMEM for the printHelp() text
init42.h deleted  initialiseEEPROMbehaviours() now in Mind.ino   
mind.ino - getMind(), saveMind(), initialiseEEPROM(), initBehaviour8(), initialiseEEPROMbehaviours1to7()  
system42.ino -> deleted -  disable_walking(), enable_walking(), toggle_f_walking() moved to behaviour.ino
                            EEPROM initialisation now in Mind.ino 
varsCons45.h  EEPROM variables renamed, slot variables added, char autoAct ='0' ;
/*== CHANGED ================================================
INCLUDES
deleted - #include <avr/pgmspace.h>     // for PROGMEM for the printHelp() text

void setup()  - the following are now setup from default Behaviour in EEPROM, see init45.h
  deleted - rollspeed =1;     // start off slow
  deleted - pacespeed =1;     // start off slow
  deleted - rollspeedmax =4;  // quite fast 
  deleted - pacespeedmax =6;  // quite fast  
  deleted - f_walking =EEPROM.read(Ef_walking);
  deleted - toggle_f_walking();       // toggles on reset
  deleted - EEPROM.write(BehaviourEbase +Es31f_walking,f_walking);   // save new state

void loop()
      Ccmnd =autoAct;
    case 'b':   if (f_recording) { RecordCcmnd(); }    //so 'b' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { case '0': // fall through
                  case '1': // fall through
                  case '2': // fall through
                  case '3': // fall through
                  case '4': // fall through
                  case '5': // fall through
                  case '6': // fall through
                  case '7': // fall through
                  case '8': if(Ccmnd<'0' || Ccmnd>'8') { Ccmnd ='?'; break; }
                            if (f_recording) { RecordCcmnd(); }      //so Ccmnd is recorded
                            Behaviour =Ccmnd;
							              if(Behaviour<>'0') 
							              {  EEPROM.write(0 +BehaviourEbase,'.');     //deselect old behaviour 
						                   BehaviourEbase =(Behaviour -'1') *32;    //new behaviour 
                               EEPROM.write(0 +BehaviourEbase,'@');     //new behaviour is current
                               }  
							              getMind()
                            break;
                  case 's': if (f_recording) { RecordCcmnd(); }          //so 's' is recorded
                            GetCcmnd();                                  //get parameter
                            if(Ccmnd<'0' || Ccmnd>'8') { Ccmnd ='?'; break; }
							if (f_recording) { RecordCcmnd(); }          //so Ccmnd is recorded
                            if(Ccmnd=='0') {slotbase =BehaviourEbase;} else {slotbase =(Ccmnd -'1') *32;}
                            saveMind();                 // save current behaviour to a behaviour slot
                            break;
                  case '#': //deleted saving of rname and rgroup to EEPROM, now use bs,1-8
                  case 'a': if (f_recording) { RecordCcmnd(); }    //so 'p' is recorded
                            autoAct =Ccmnd;
                  case 'u': 
                            case 's': EEPROM.write(BehaviourEbase +ESonarAwareAt,SonarAwareAt);
                                      EEPROM.write(BehaviourEbase +ESonarTooNear,SonarTooNear);
    case '?':        case 'B':  PrintBehaviours(); break;                            
void printBehaviour()
  Serial.print(Behaviour);     Serial.println(" Behaviour"); 
  Serial.print(autoAct);       Serial.println(" autoAct");      
  
AdjustServos45.ino
  adjusted EEPROM.write and EEPROM.read to include BehaviourEbase so correct slot is written to
  
behaviour44 -> behaviour45 - added disable_walking(), enable_walking(), toggle_f_walking()  
 
//== HELP TEXT
const char h_bb[]   PROGMEM = "'b','0'-'8' - set current default behaviour and copy to RAM BeHaves
const char h_bn[]   PROGMEM = "    's','1'-'8' - save current RAM BeHaviour to EEPROM Behaviour
const char h_bl[]   PROGMEM = "    'l','1'-'8' - load behaviour slot 1-8";
const char h_qB[]   PROGMEM = "   '?B' - print all Behaviours in EEPROM";

//== USER ROUTINES 
New section created after the end of void loop{}
and the following moved there
void Experiment()      //until Quit
void Xplore()          //Wander() until Quit
void beAlive()         // the loop() from B_PM 
*/
//== INCLUDES ===============================================
#include "pins20.h"
#include "varsCons45.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
  
//== VARIABLES and CONSTANTS ================================
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play current Act
boolean f_reacting =false;    //in a ReAct - set in behaviour44.ino if a React is initiated.
boolean f_doDefault =true;    //true for the first loop and #< only
boolean f_runDefault =false;  //run Default when nothing else to do
const byte NL =10;            //NewLine value
const byte LF =10;            //LineFeed value                                  
const byte CR =13;            //Carriage Return value
const byte Esc =27;           //Esc key value
char Act;                     //current Act as '1' - '8'
byte actIndex =0;             //current Act number, set to default           
int EEPROMindex =-1;          //index to moves in an Act in EEPROM, -1 => nothing to continue with #>
int hashhashAt;               //copy of EEPROMindex at hashhashSTOP 
int ReactAt;                  //copy of EEPROMindex at React
int ActEnd;                   //end address of current recording Act in EEPROM
int moves =0;                 //current move
char Ccmnd =0;                //current Character command, default null command
byte epage =1;                //the Act page for the current Act, default page =1 for  Acts 1-8 and page =2 for 01-08
char Behaviour;               //read from EEPROM in init, current behaviour 1-8 in epage 0
byte BehaviourEbase =224;     //base address of current behaviour 0,32,64,96,128,160,192,(224)
byte slotbase;                //working slotbase so current BehaviourEEbase is not changed on save
byte behaveNum;               //BeHave address in Behaviour slot 
byte behaveValue;
String numString ="";         //temporary string in loadBehaviour(), getnum()

//== SETUP ==================================================
void setup() 
{ 
  f_BeQuiet =0;                    // send status messages
  f_MsgForMe =1;                   // listen to messages   
  f_hashhashSTOP =0;               
  
  Serial.begin(9600);      // to communicate with Serial Monitor 
      
  pinMode(voicePin,OUTPUT);
  pinMode(LbeardLEDpin_, OUTPUT);  
  pinMode(RbeardLEDpin_, OUTPUT);  
  pinMode(LwhiskerLEDpin_, OUTPUT);
  pinMode(RwhiskerLEDpin_, OUTPUT);
  digitalWrite(LbeardLEDpin_, HIGH);  // beard LED OFF
  digitalWrite(RbeardLEDpin_, HIGH);  // beard LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  
  if (EEPROM.read(E255f_walking)>1 ) {initialiseEEPROM();}   // in case it is not initialised
  
  //find current current behaviour in EEPROM and copy to RAM
  for (BehaviourEbase =0; BehaviourEbase < 224; BehaviourEbase +=32 )  //test first 7 slots 
  {if (EEPROM.read(BehaviourEbase) =='@')  break; }
  if (EEPROM.read(BehaviourEbase) !='@') { EEPROM.write(BehaviourEbase,'@'); }    //if '@' not found at 8 make 8 the default 
  Behaviour =BehaviourEbase /32 +'1';
  getMind();
    
  Serial.print(thisrobot); // so we know which robot the program is for
  Serial.print(" ");
  Serial.print(thisprog);  // so we know which program is running
  Serial.print(" ");
  Serial.print(rgroup);    // so we know which group it is in
  Serial.println(rname);   // so we know which robot this is

  //now configure Bambino
  f_forwards =1;    // set default

  pinMode(servoRollPin, OUTPUT);    // sets the digital pin as output
  pinMode(servoPacePin, OUTPUT);    // sets the digital pin as output
  digitalWrite(servoRollPin, LOW);   // initialise pin
  digitalWrite(servoPacePin, LOW);   // initialise pin

  if (f_walking) { Serial.println(" walking"); } else { Serial.println(" stopped"); }

  ServosInitC();
  Srollat =rollat;
  Spaceat =paceat;
  
}
//== LOOP ===================================================
void loop()
{ 
  f_recordC =true;     // record Command, default =true
  
#ifdef BambinoMM 
  redLEDdim();         //indicate ready for command except    
  if(f_recording)  { redLEDon(); }
  if(f_play)       { redLEDoff(); }
#else
  if(f_recording)  { redLEDon(); } else { redLEDoff(); }
  if(!f_reacting)                    // don't change if in a ReAct
    { if(f_play)
        { whiskerLEDsOff();  }
        else
        {  whiskerLEDsDim(); }    //indicate ready for command
      }  
#endif      

  if(!f_play && !f_reacting && !f_BeQuiet){Serial.print(">"); }    //ready prompt
  
  if (f_doDefault)   // true on the first loop and #< only
    { f_doDefault =false;                // don't do it again
      Ccmnd =autoAct;
      if(Ccmnd=='0') { f_runDefault =false;  Ccmnd ='.'; }     // '0' -> no autorun, can't run it so do stop 
    }   
   else
    // if f_hashhashSTOP skip GetCcmnd() and process Ccmnd '.' from ##
    { if(!f_hashhashSTOP) { GetCcmnd(); } }   //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd

DoCommand:
  switch (Ccmnd) 
  { case LF:    f_recordC =false;  break;     //linefeed
    case CR:    f_recordC =false;  break;     //carriage return
    case ' ':   break;     //strip space
    case ',':   break;     //strip comma
    case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
    case '.':   if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }     //terminated string
                f_recording =false;
                if(!f_reacting) 
                  { f_play =false;                                                        // end of Act  
                    if (!f_hashhashSTOP) { EEPROMindex =-1; };   // so there is nothing to continue with #>
                    f_hashhashSTOP =false;
                    if(!f_BeQuiet){Serial.println();}
                    if (f_runDefault) { f_doDefault =true; }
                  } 
                if(f_reacting) { f_reacting =false; EEPROMindex =ReactAt; beardLEDsOff(); }     // end of ReAct
                break;

    case 'A':	  // if recording record 'A' otherwise do beAlive() until 'Q' or reset
                if (!f_recording) { beAlive(); }  break;
    case 'X':	  // if recording record 'X' otherwise do Xplore() until 'Q' or reset
                if (!f_recording) { Xplore(); }  break; 
    case 'E':	  // if recording record 'E' otherwise do Experiment() until 'Q' or reset
                if (!f_recording) { Experiment(); }  break; 
    case '!':   epage =3;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '0':   epage =2;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '1':   // fall through
    case '2':   // fall through
    case '3':   // fall through
    case '4':   // fall through
    case '5':   // fall through
    case '6':   // fall through
    case '7':   // fall through
    case '8':   if (!f_recording) { f_play =true; Act =Ccmnd;  setEEPROMindexes(Act); }  break;
    case 'F':   forward();      break;        
    case 'B':   backward();     break;        
    case 'R':   turnRight();    break;        
    case 'L':   turnLeft();     break;        
    case 'H':   halt();         break; 
    case 'V':   whee();         break;   //Spkr and LED on different pins           
    case 'S':   f_recordC =false;
                toggle_f_walking(); 
                if(!f_BeQuiet)
                  {if (f_walking) {Serial.println(" walking");} else {Serial.println(" stopped");}}
                break;    
    case 'U':   USreact();      break; 
    case 'W':   WhiskerReact(); break; 
    case 'G':   BeardReact();   break;
    case 'v':   if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd) 
                { case 'b':  bip();   break;
                  case 'e':  whee();  break;
                  case 'o':  whoa();  break;
                  default:   bip();   Ccmnd ='b';  break;
                }    // end switch
                break;                
    case 'w':   if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded
                GetCcmnd();                            //get parameter
                makeCcmndDigit();                      //ensure '1'-'9'
                waitCcmndSecs();
                break;                                 // if recording Ccmnd will be recorded
    case 's':   if (f_recording) { RecordCcmnd(); }    //so 's' is recorded
                GetCcmnd();                            //get parameter
                if (Ccmnd=='u') { speedup();  break; }     // and if recording record 'u' 
                if (Ccmnd=='d') { speeddown();  break; }   // and if recording record 'd' 
                if (Ccmnd=='m') { rollspeedmax =rollspeed; pacespeedmax =pacespeed; break; }
                if (Ccmnd=='r') 
                { if (f_recording) { RecordCcmnd(); }  //so 'r' is recorded
                  GetCcmnd();                          //get parameter
                  if (Ccmnd=='m') { rollspeedmax =rollspeed; break; }
                  //else                 
                  makeCcmndDigit();                    //ensure '1'-'9'    
                  rollspeed =Ccmnd -'0';
                  break;                               // and if recording record digit
                }
                if (Ccmnd=='p') 
                { if (f_recording) { RecordCcmnd(); }  //so 'p' is recorded
                  GetCcmnd();                          //get parameter
                  if (Ccmnd=='m') { pacespeedmax =pacespeed; break; }
                  //else                 
                  makeCcmndDigit();                    //ensure '1'-'9'    
                  pacespeed =Ccmnd -'0';
                  break;                               // and if recording record digit 
                }
                //else                 
                makeCcmndDigit();                      //ensure '1'-'9'    
                rollspeed =max(1,Ccmnd -'0'); 
                pacespeed =max(1,Ccmnd -'0');       	
                break;                                 // and if recording record digit 
    case 't':   if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { case 'a':  turnAway();    break;
                  case 'm':  turnMemory();  break;
                  case 'o':  turnOther();   break;
                  case 'l': turnLight();   break;
                  case 'd': turnDark();    break;
                  case '@': if (f_recording) { RecordCcmnd(); }    //so '@' is recorded
                            GetCcmnd();                            //get parameter
                            switch (Ccmnd)
                            { case 'l':  turnAtLight();  break;
                              case 'd':  turnAtDark();   break;
                              default:   badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                                         Ccmnd ='?';     //so if recording then '?' is recorded
                                         break;          //don't do anything 
                            }    // end switch
                  default:  turnMemory();   Ccmnd ='m';  break;
                }     // end switch case t
                break;     
//------------                 
    case 'b':   if (f_recording) { RecordCcmnd(); }    //so 'b' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { 
  b_behaviour:    case '0': if (f_recording) { RecordCcmnd(); }      //so Ccmnd is recorded
                            getMind();  break;
                  case '1': // fall through
                  case '2': // fall through
                  case '3': // fall through
                  case '4': // fall through
                  case '5': // fall through
                  case '6': // fall through
                  case '7': // fall through
                  case '8': if (f_recording) { RecordCcmnd(); }      //so Ccmnd is recorded
                            Behaviour =Ccmnd;
                            EEPROM.write(0 +BehaviourEbase,'.');     //deselect old behaviour 
                            BehaviourEbase =(Behaviour -'1') *32;    //new behaviour 
                            EEPROM.write(0 +BehaviourEbase,'@');     //new behaviour is current
                            getMind();
                            break;
                  case 's': if (f_recording) { RecordCcmnd(); }          //so 's' is recorded
                            GetCcmnd();                                  //get parameter
                            if(Ccmnd<'0' || Ccmnd>'8') { Ccmnd ='?'; break; }
                            if (f_recording) { RecordCcmnd(); }          //so Ccmnd is recorded
                            if(Ccmnd=='0') {slotbase =BehaviourEbase;} else {slotbase =(Ccmnd -'1') *32;}
                            saveMind();                 // save current behaviour to a behaviour slot
                            break;
                  case 'l': if (f_recording) 
                              { Ccmnd ='.'; RecordCcmnd();  //so '.' is recorded
                                f_recording =false;
                                break;
                               }
                            if (!f_recording) {loadBehaviour();}
                            break;	
                  case '#': if (f_recording) { RecordCcmnd(); }    //so '#' is recorded                            
                            GetCcmnd();                     //get parameter
                            if( f_BeQuiet==1 ) { break; }     // don't change these if being quiet
                            if (Ccmnd>='0' && Ccmnd<='9')
                              { rgroup =Ccmnd;  EEPROM.write(rgroup,rgroup);  break; }
                            if (Ccmnd>='A' && Ccmnd<='z') 
                              { rname =Ccmnd;  EEPROM.write(rname,rname);  break; }
                            badkey();  break;
                  case 'a': if (f_recording) { RecordCcmnd(); }    //so 'a' is recorded
                            GetCcmnd();                            //get parameter
                            if(Ccmnd<'0' || Ccmnd>'9') { Ccmnd ='0'; break; }
                            if (Ccmnd=='9') {f_runDefault =true; break;}             // ## sets f_runDefault false
                            autoAct =Ccmnd;
                            break;
                  case 'w': if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded 
                            GetCcmnd();                            //get parameter
                            if (Ccmnd=='0') { disable_walking();  break; } // and if recording record '0' 
                            if (Ccmnd=='1') { enable_walking();  break; }   // and if recording record '1' 
                            Ccmnd ='?';   // bad char, no action and if recording record '?' 
                            break;                                 // if recording Ccmnd will be recorded
                  case 'p': if (f_recording) { RecordCcmnd(); }    //so 'p' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            paceby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;
                  case 't': if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            turnby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;
                  case 'r': if (f_recording) { RecordCcmnd(); }    //so 'r' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '1'-'9'
                            rollby =(Ccmnd -'0') *2;  //gives a maximum of 18, default 15
                            break;         
                  case 'u': if (f_recording) { RecordCcmnd(); }    //so 'u' is recorded
                            GetCcmnd();                             //get parameter
                            switch (Ccmnd)
                            { case 'm': // don't record 'm'
                                        do
                                        { Serial.println("press [+] for AwareAt, [-] for TooNear, [.] or [Esc] to Quit");
                                          GetCcmnd();                            //get parameter
                                          objectRange =dist_HC_SR04(); if(f_cm) {objectRange /=5;} else {objectRange /=2;}
                                          if (Ccmnd=='+') 
                                            { SonarAwareAt =objectRange; 
                                              if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                              Ccmnd ='0' +SonarAwareAt; if (f_recording) { RecordCcmnd(); }    //so 'SonarAwareAt' is recorded
                                              }
                                          if (Ccmnd=='-')
                                            { SonarTooNear =objectRange;
                                              if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                              Ccmnd ='0' +SonarTooNear; if (f_recording) { RecordCcmnd(); }    //so 'SonarTooNear' is recorded
                                              }
                                        } while (Ccmnd!='.' &&  Ccmnd!='Q' && Ccmnd!=Esc);
                                        break;
                              case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarAwareAt =Ccmnd -'0';
                                        break;
                              case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '1'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarTooNear =Ccmnd -'0';
                                        break;
                              case 'c': if (f_recording) { RecordCcmnd(); }    //so 'c' is recorded
                                        GetCcmnd();                            //get parameter
                                        if((Ccmnd -'0') ==1) { f_cm =true; } else { f_cm =false; }
                                        break;
                              case 's': EEPROM.write(BehaviourEbase +Es19SonarAwareAt,SonarAwareAt);
                                        EEPROM.write(BehaviourEbase +Es18SonarTooNear,SonarTooNear);
                                        break;
                            }     // end switch
                            break;
                  case 'q': f_BeQuiet =1;  break;           
  b_verbose:      case 'v': if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded          
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='1') { f_bv =1; }
                            else 
                            { Ccmnd ='0'; f_bv =0; }               //default '0'   
                            break;
                  case '@': if( f_BeQuiet==1 ) { break; }     // don't adjust if being quiet
                            AdjustServos();  break;           
                  default:  badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                            Ccmnd ='?';     //so if recording then '?' is recorded
                            break;          //don't do anything
                }     // end switch
                break;
//------------end case 'b'
                
    case 'r':   f_recordC =false;
                redLEDon();
                if (!f_recording) 
                  { getAct();  if(Ccmnd=='.') { break; }  
                  setEEPROMindexes(Act);  f_recording =true; 
                  }
                break;
    case 'c':   f_recordC =false;  if (!f_recording) { copyAct(); }  break;	
    case 'l':   f_recordC =false;  if (!f_recording) { loadAct(); }  break;	
    case '?':   f_recordC =false;
                if (!f_recording) 
                  { GetCcmnd();                           //get parameter
                    if( f_BeQuiet==1 ) { break; }         // don't print if being quiet
                    switch (Ccmnd)
                    { case 'h':  printHelp();  break;     // print instructions
                      case '1':  // fall through
                      case '2':  // fall through
                      case '3':  // fall through
                      case '4':  // fall through
                      case '5':  // fall through
                      case '6':  // fall through
                      case '7':  // fall through
                      case '8':  Act =Ccmnd;  printAct();  break;
                      case 'p':  PrintAllActs();  break;
                      case '?':  PrintAllActs();  break;
                      case 'd':  DumpActs();  break;
                      case 'b':  printBehaviour();  break;
                      case 'm':  printMind();  break;      // - all Acts, Behaviour
                      case '!':  PrintReActs(); break;
                      case 'B':  PrintBehaviours(); break;
                      default:   badkey();  if(!!f_BeQuiet){Serial.println("?");}  break;
                    }    // end switch               
                  }    // end if 
                break;
    case '#':   f_recordC =false;
                f_MsgForMe =1;
                f_hashcmnd =true;           // don't get a Play command here
                GetCcmnd();                 //get parameter
                f_hashcmnd =false;
                switch (Ccmnd)
                { case '*': f_MsgForMe =1;  f_BeQuiet =1;  hashgroup =rgroup;  break;  //everybody listen
                  case '#': f_MsgForMe =1;  f_BeQuiet =0;                  //everybody stop
                            disable_walking();  Ccmnd ='.';  hashhashAt =EEPROMindex;
                            f_hashhashSTOP =true;  // so '.' gets processed
                            f_runDefault =false;
                            break;
                  case '<': f_doDefault =true;  f_hashhashSTOP =false;  enable_walking();  break;
                  case '>': if (hashhashAt!=-1)                //something to continue
                            { f_play =true;  enable_walking(); EEPROMindex =hashhashAt; }
                            f_hashhashSTOP =false;    
                            break;
                  case '?': delay((rgroup -'0')*50+(rname-'A')*10);
                            Serial.print(" "); Serial.print(rgroup); Serial.print(rname); 
                            break;
		  case '.': f_MsgForMe =0;  f_BeQuiet =1;  hashgroup =' ';     //end #session
				                    break;
                  default:  if (Ccmnd>='A' && Ccmnd<='Z' ) { 
                              hashgroup =Ccmnd; 
                              if (hashgroup==rgroup) 
                                { f_MsgForMe =1;  f_BeQuiet =1;  }    //my group
                               else
                                { f_MsgForMe =0;  f_BeQuiet =1;  }    //not my group
                              break;
                              }
                            if (Ccmnd==rname && hashgroup==rgroup)  { f_MsgForMe =1;  f_BeQuiet =0;  break; }  //me
                            f_BeQuiet =1;      // any other key             //not for me 
                            break;
                }
                break;              
                
    default:    f_recordC =false;    //don't record bad key		
                badkey();  if(!f_BeQuiet){ Serial.print("bad key ");}  break;
  }    // end switch
	
Record:	
  if (f_recording  && f_recordC) { RecordCcmnd(); } 
  
}
//== END {LOOP} =============================================

//== USER ROUTINES ==========================================
void Experiment()       //until Quit
{ while(true)
  { 
    //put your own routine here e.g.
    rollspeed =4;
    pacespeed =4;
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    whee(); 
    turnRight();
    turnRight();
    turnRight();    
    whee(); 
    rollspeed =2;
    pacespeed =2;
    turnLeft();
    turnLeft();
    turnLeft();
    whee(); 
    rollspeed =1;
    pacespeed =1;
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    whee();    
    whee();    
    whee();    
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------
void Xplore()         //Wander() until Quit
{ while(true)
  { Wander();
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------
void beAlive()         // the loop() from B_PM22_BeAlive
{ while(true)
  { switch (random(9))
    { case 0: WanderSteps(random(10)); break;    //uses all sensors
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
    } // end switch
    if( Quit() ) { break; }
  }	// end while
}

//== FUNCTIONS ==============================================

void GetCcmnd()	       //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
{ Ccmnd =0;
  while(Ccmnd==0)
  { if (f_recording)  
    { while(!Serial.available()) {}       //wait for Serial
      Ccmnd =Serial.read();  bip(); 
    } 
    else         //if(!f_recording)
    { if (Serial.available()) 
      { Ccmnd =Serial.read();            //over-ride if Serial, null ignored
        if (!f_MsgForMe && Ccmnd!='#') { Ccmnd =0; }     //discard messages for others
        else { bip(); }
      }  
      else 
      { if (!f_hashcmnd)         //don't get a Play character if in the middle of a hash command   
        { if (f_play || f_reacting)
          { Ccmnd =PlayAct();             //get Act or React Ccmnd 
          }
        }  
      }  //end if (!f_hashcmnd)
    }  //end if (!f_recording)                                               
  }  //end while Ccmnd==0
  if ( !f_BeQuiet ){ Serial.print(Ccmnd); }
}   
//----------------------------
void RecordCcmnd()	
{ EEPROM.write(EEPROMindex,Ccmnd);
  EEPROMindex++;
  if (EEPROMindex>=ActEnd) { f_recording =false;  EEPROM.write(ActEnd,'.');  badkey(); }     //terminated string 
}  
//----------------------------
void makeCcmndDigit()      //ensure 1-9
{ if (Ccmnd<'1' || Ccmnd>'9') { Ccmnd ='1'; } //default
}
//----------------------------  
void waitCcmndSecs()   //read delay time from Act and wait
{ if (Ccmnd<'1' || Ccmnd>'9') { Ccmnd ='1'; } //default
  byte cmnd =Ccmnd -'0';                          // ASCII to numeric
  delay(cmnd *1000);
}  
//----------------------------  
void printMind()
{ PrintAllActs();
  printBehaviour();
}
//----------------------------
void PrintReActs()
{ Serial.println();  Serial.print("W & G, 1 indicates object or hole sensed LR");
  for(Act ='1'; Act<='8'; Act++) { epage =3; printReAct(); }  Serial.println(); 
}
//----------------------------
void printReAct()
{ Serial.println();
  Serial.print("ReAct");  Serial.print(Act);  Serial.print(" ");
  switch (Act)
  { case '1':  Serial.print("U<=");  Serial.print(SonarAwareAt);  Serial.print(" ");  break;
    case '2':  Serial.print("U<=");  Serial.print(SonarTooNear);  Serial.print(" ");  break;
    case '3':  Serial.print("W11");  Serial.print("  ");  break;
    case '4':  Serial.print("W10");  Serial.print("  ");  break;
    case '5':  Serial.print("W01");  Serial.print("  ");  break;
    case '6':  Serial.print("G11");  Serial.print("  ");  break;
    case '7':  Serial.print("G10");  Serial.print("  ");  break;
    case '8':  Serial.print("G01");  Serial.print("  ");  break;
  } // end switch
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);    //there are moves which take more than 1 character  
  Serial.print(" bytes"); //so the space taken in EEPROM is really bytes
  moves =0;	
}
//---------------------------- 
void loadBehaviour()
{ Behaviour ='0';  getBehaviour();  // 0 - 9
  while( Behaviour!='0' )
  { Serial.println("BeHave number,BeHave value; - Esc or '.' to quit");
    do                                  //get BeHave address in the slot, ended with ','
    { Ccmnd =' ';
      if(Serial.available()) { Ccmnd =Serial.read(); Serial.print(Ccmnd);  bip(); }
      if (isDigit(Ccmnd)) { numString += Ccmnd; Ccmnd =' '; }
       //[Return] is \r \n so let isDigit filter \r      
      }while( Ccmnd!='\n' && Ccmnd!=',' && Ccmnd!='.' && Ccmnd!=Esc ); 
      
    if(Ccmnd=='.' || Ccmnd==Esc) break;             //exit while

    behaveNum =numString.toInt(); numString ="";
    
    switch (behaveNum)
    { case 31:  //Ef_walking    - true, false
                get1char(); behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 30:  //ErollC        - ~90
                getnum();   behaveValue =constrain(behaveValue,82,108);  
                break;
      case 29:  //EpaceC        - ~90
                getnum();   behaveValue =constrain(behaveValue,82,108);  
                break;
      case 28:  //Erollby       - 1-15
                getnum();   behaveValue =constrain(behaveValue,1,18);  
                break;
      case 27:  //Epaceby       - 1-15 
                getnum();   behaveValue =constrain(behaveValue,1,18);  
                break;
      case 22:  //Ergroup       - '0-9'
                get1char();  behaveValue =constrain(Ccmnd,'0','9');
                break;
      case 21:  //Ername        - 'A-z'
                get1char();  behaveValue =constrain(Ccmnd,'A','z');
                break;
      case 20:  //EautoAct      - '0-8'
                get1char();  behaveValue =constrain(Ccmnd,'0','8');
                break;
      case 19:  //ESonarAwareAt - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 18:  //ESonarTooNear - 1-9
                get1char(); behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 17:  //Ef_cm         - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 16:  //Erollspeedmax - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 15:  //Epacespeedmax - 1-9
                getchar();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 14:  //Erollspeed    - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 13:  //Epacespeed    - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 12:  //f_bG    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 11:  //f_bW    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 10:  //f_bU    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 9:   //f_AutoSense    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      default:  behaveNum =0;
                Serial.print("bad BeHave number");  Serial.println();  
                break;      
      }   //end switch
    if(behaveNum!=0) { 
      if(Behaviour!='9') { 
        BehaviourEbase =(Behaviour -'1') *32; 
        EEPROM.write(BehaviourEbase +behaveNum, behaveValue);
       }   
       else {
         for ( Behaviour =1; Behaviour <9; Behaviour ++ )  {
           BehaviourEbase =Behaviour *32; 
           EEPROM.write(BehaviourEbase +behaveNum, behaveValue);
          }
         Behaviour ='9'; 
        }
      }  
    } //end while
}
//---------------------------- 
char get1char()
{ Ccmnd =0;
  do{ if(Serial.available())  Ccmnd =Serial.read(); }while(Ccmnd==0);
  Serial.println(Ccmnd);  bip();
}
//----------------------------  
byte getnum()
{ do
  { Ccmnd =' ';
    if(Serial.available()) { Ccmnd =Serial.read();  Serial.print(Ccmnd);  bip();} 
    if (isDigit(Ccmnd)) { numString +=Ccmnd;   Ccmnd =' '; }
       //[Return] is \r \n so let isDigit discard \r      
    }while(Ccmnd!='\n' && Ccmnd!='.' && Ccmnd!=',' && Ccmnd!=';' && Ccmnd!=Esc);
  behaveValue =numString.toInt();  numString = "";
  if(Ccmnd=='.' || Ccmnd==Esc) { Serial.println();  behaveNum =0; }
}
//----------------------------  
char getBehaviour()      //called when loading a behaviour, Ccmnd ->'.', '1' - '8'
{ Ccmnd =0;
  do
  { if(Serial.available()) 
    { Ccmnd =Serial.read();
      switch (Ccmnd) 
      { case Esc:  Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  break;           //quit
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':  Behaviour =Ccmnd;  break;
        default:   badkey(); Serial.print(" enter Behaviour or '.'"); Ccmnd =0;
      }  //end case
    } //end if Serial
  } while(Ccmnd==0);
  Serial.println(Ccmnd);
}    
//----------------------------  
void printBehaviour()
{ Serial.println();                
  Serial.println(thisprog); // so we know which program is running
  Serial.print("robot ");      Serial.print(rgroup);   Serial.print(rname); 
                               Serial.print(" MsgForMe=");  Serial.print(f_MsgForMe); 
                               Serial.print(" BeQuiet=");   Serial.println(f_BeQuiet); 
  Serial.print(Behaviour);     Serial.println(" Behaviour"); 
  Serial.print(f_walking);     Serial.println(" walking"); 
  Serial.print(autoAct);       Serial.print(" autoAct,  "); Serial.print("rerun "); if (f_runDefault==1) { Serial.print("On"); } else { Serial.print("Off"); } Serial.println("  (default Off)"); 
  Serial.print(rollspeed);     Serial.println(" rollspeed");
  Serial.print(pacespeed);     Serial.println(" pacespeed");
  Serial.print(rollspeedmax);  Serial.println(" rollspeedmax");
  Serial.print(pacespeedmax);  Serial.println(" pacespeedmax");  
  Serial.print(rollby);        Serial.print('(');        Serial.print(rollby/2);        Serial.print(')');        Serial.println(" rollby");
  Serial.print(paceby);        Serial.print('(');        Serial.print(paceby/2);        Serial.print(')');        Serial.println(" paceby");
  Serial.print(turnby);        Serial.print('(');        Serial.print(turnby/2);        Serial.print(')');        Serial.println(" turnby");
  Serial.print(rollC);         Serial.println(" rollC");    
  Serial.print(paceC);         Serial.println(" paceC");   
  Serial.print(SonarAwareAt);  Serial.println(" SonarAwareAt");   
  Serial.print(SonarTooNear);  Serial.println(" SonarTooNear");   
  Serial.print(f_bv);          Serial.println(" verbose");
//disp_freeram();  
}
//----------------------------  
void PrintBehaviours()
//This uses Serial.print(F(" text ")); which saves RAM by leaving the text in Program memory 
//and is a lot easier to use than the PROGMEM construct used for the help text - we can learn something every day!
{ Serial.println();
  Serial.println(F("BEHAVIOUR       b1    b2    b3    b4    b5    b6    b7    b8"));
  Serial.println(F(" # BeHave       --    --    --    --    --    --    --    --"));
  Serial.print(F("31 f_walking    "));
    for(int i =31; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0=false, 1=true");
  Serial.print(F("30 rollC        "));
    for(int i =30; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  Serial.print(F("29 paceC        "));
    for(int i =29; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  Serial.print(F("28 rollby       "));
    for(int i =28; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  Serial.print(F("27 paceby       "));
    for(int i =27; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  //23-26 never altered
  Serial.print(F("22 rgroup       "));
    for(int i =22; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("A-Z");
  Serial.print(F("21 rname        "));
    for(int i =21; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("0-9");
  Serial.print(F("20 autoAct      "));
    for(int i =20; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("0-8");
  Serial.print(F("19 SonarAwareAt "));
    for(int i =19; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0-9, units 5cm or 2inch");
  Serial.print(F("18 SonarTooNear "));
    for(int i =18; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0-9, units 5cm or 2inch");
  Serial.print(F("17 Sonar use cm "));
    for(int i =17; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1=true, 0=inch");
  Serial.print(F("16 rollspeedmax "));
    for(int i =16; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("15 pacespeedmax "));
    for(int i =15; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("14 rollspeed    "));
    for(int i =14; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("13 pacespeed    "));
    for(int i =13; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("-- @Behaviour   "));
    for(int i =0; i<256; i +=32) { Serial.print(char(EEPROM.read(i)));  Serial.print(F("     ")); } Serial.println();
} 
//----------------------------  
void DumpActs()        //Dump Acts, Acts0, ReActs. Print without names and length
{ for(Act ='1'; Act<='8'; Act++)
  { setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =2;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =3;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  Serial.println();
}		
//----------------------------  
void loadAct()
{ redLEDon();  
  getAct();                         // Ccmnd ->'.' - Quit,
                                    // '0', '!'    - sets Act0 or ReAct page then gets Act
                                    // '1' - '8'   - sets Act page'1' - '8',  - 
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    while(true) 
    { while(!Serial.available()) {}  //wait for Serial
      Ccmnd =Serial.read();  Serial.print(Ccmnd);
      if(Ccmnd=='.' || Ccmnd==Esc) { break; }
      EEPROM.write(EEPROMindex,Ccmnd);  EEPROMindex++;
      if (EEPROMindex>=ActEnd) { badkey();  break; }     //ActEnd =EEPROMindex +31;   
    }
    EEPROM.write(EEPROMindex,'.');                        //terminated string 
  }
}
//----------------------------  
void copyAct()
{ getAct();
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    int fromEEPROMindex =EEPROMindex;  
    int fromActEnd =ActEnd;
    getAct();
    if (Ccmnd!='.')
    { setEEPROMindexes(Act);
      for(byte i =0; i<=fromActEnd -fromEEPROMindex; i++)
      { EEPROM.write(EEPROMindex +i,EEPROM.read(fromEEPROMindex +i)); }
    } 
  }	
}
//----------------------------
char PlayAct() 
{ char _Ccmnd =EEPROM.read(EEPROMindex++);
  if(_Ccmnd==255 || _Ccmnd==-1) _Ccmnd ='.';     //uninitialised EEPROM value, don't play it, change to '.'
  return _Ccmnd;
}  
//----------------------------
void PrintAllActs()
{ for(Act ='1'; Act<='8'; Act++) { epage =1; printAct(); }  Serial.println(); 
  for(Act ='1'; Act<='8'; Act++) { epage =2; printAct(); }  Serial.println(); 
}
//----------------------------
void printAct()
{ Serial.println();
  Serial.print("Act");
  if(epage==2) { Serial.print('0'); }    //for logical page0 Acts
  Serial.print(Act);
  Serial.print(" ");
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);
  Serial.print(" bytes");
  moves =0;	
}
//----------------------------  
void getAct()      //called when recording, loading, printing and copying Acts, Ccmnd ->'.', '1' - '8'
{ Ccmnd =0;
  do
  { if(Serial.available()) 
    { Ccmnd =Serial.read();   Serial.print(Ccmnd);
      switch (Ccmnd) 
      { case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  Serial.print(">");  break;     //quit, print ready prompt
        case '!':  epage =3;  Ccmnd =0;  break;  //do it again to read Act
        case '0':  epage =2;  Ccmnd =0;  break;  //do it again to read Act
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':  Act =Ccmnd;  break;
        default: badkey(); Serial.print(" enter Act or '.'"); Ccmnd =0;	
      }  //end case
    } //end if Serial
  } while(Ccmnd==0);
  //Serial.print(Ccmnd);
}    
//----------------------------
void setEEPROMindexes(char Act)     // set EEPROM pointers
{ actIndex =Act -'1';               // ASCII to numeric,  Act 1 == actIndex 0
  EEPROMindex =(256 *epage) +(actIndex*32);  
  ActEnd =EEPROMindex +31;
  epage =1;            //set to default page, Acts 1 - 8 
}
//----------------------------
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF  
  whoa();  
}  
//----------------------------
void USreact()     // "U" read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
{ objectRange =dist_HC_SR04();
  if( !f_BeQuiet ) { Serial.println(); }
  if( !f_BeQuiet ) { Serial.print("Ultrasonic distance ="); }
  if( !f_BeQuiet ) { Serial.print(objectRange); }
  if( !f_BeQuiet ) { if(f_cm) {Serial.println("cm");} else {Serial.println("inch");} }
  reactUObjects();
}
//----------------------------
void WhiskerReact()     // "W" read Whisker sensors and avoid obstacles (if any) using avoidObstacles() behaviour rules
{ readWhiskers(); 
  queryObstaclesLR();
  reactObstaclesLR();
}
//----------------------------
void BeardReact()     // "G" Ground - read Beard sensors and avoid holes (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdgeLR();
  reactHolesLR(); 
}
//---------------------------
boolean Quit()          //check for Quit
{ if( Serial.available() ) 
  { Ccmnd =Serial.peek(); 
    if( Ccmnd=='.' || Ccmnd==Esc || Ccmnd=='#')
      { return true; }
     else
      { return false; }
  }  
}
//----------------------------
void bip()            // make a bip sound
{ beep(voicePin,1500,50); }
//----------------------------
#ifdef BambinoMM
void redLEDdim()
{ analogWrite(redLEDpin_,254); }  //redLED, LOW=on so 255 is off 
//----------------------------
void redLEDon()
{ digitalWrite(redLEDpin_,LOW); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin_,HIGH); }  
//---------------------------- 
//BambinoMM does not have Whisker sensors
//---------------------------- 
#else
void redLEDon()
{ digitalWrite(redLEDpin,HIGH); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin,LOW); }  
//----------------------------
void whiskerLEDsOff()
{ digitalWrite(RwhiskerLEDpin_, HIGH);     // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);     // whisker LED OFF
  }  
//----------------------------  
void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
  }
#endif
//----------------------------  
void beardLEDsOff()
{ digitalWrite(RbeardLEDpin_, HIGH);       // beard LED Off
	digitalWrite(LbeardLEDpin_, HIGH);       // beard LED Off
  }
//===========================================================
/*****************Dan*************************/
// print integer value. 
/*void i_printf( int val ) {  Serial.print(val);  }

// print string s1. 
void s_printf( char *s1 ) {  Serial.print(s1);  }
  
/*********************************************/
/*void disp_freeram()
{
  s_printf("-SRAM left: ");
  i_printf( freeRam() );
}
/*********************************************/
/*int freeRam() 
{
  extern int __heap_start,*__brkval;
  int v;
  return (int)&v - (__brkval == 0  
    ? (int)&__heap_start : (int) __brkval);  
}  
/*********************************************/
