// Bambino
// TAB FILE > sense44.ino

//     SENSE - readUSdistance(), showUSdistance(), dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels(), showObstacles() 
//           - readBeard(), QueryEdgeLR(), showBeardLightLevels(), showHoles()
//           - readLight()
//===========================================================
//NEW
//readUSdistance(), showObstacles(), showHoles(),  f_bv control of show light levels
//CHANGED
//showWhiskerLightLevels(), showBeardLightLevels()
//
//== SENSE ==================================================
void readUSdistance()         //get range to object
{ Udist =dist_HC_SR04();
  // change Udist to Uunits (5cm, 2inch)
  if (f_cm) { objectRange =Udist /5; } else { objectRange =Udist /2; }
  if (!f_BeQuiet && f_bv) { showUSdistance(); }
  if (!f_BeQuiet) { showUSobstacle(); }
  }	  
//-----------------------------------------------------------     
void showUSdistance()
{ Serial.print("[U");  Serial.print(UStime);  Serial.print("us=");
  if (f_cm) { Serial.print(UStime/uscm); } else { Serial.print(UStime/usinch); }
  if (f_cm) {Serial.print("cm]");} else {Serial.print("inch]");} 
}   
//-----------------------------------------------------------     
int dist_HC_SR04()              //get range to object        //2023-02-15
{ //int UStime; is global - varsCons47.h
  pinMode(pingPin,OUTPUT);
  digitalWrite(pingPin,HIGH);
  delayMicroseconds(5);                  // 3 and below gives erratic results
  digitalWrite(pingPin,LOW);
  pinMode(pingPin,INPUT);
  UStime =pulseIn(pingPin,HIGH,UStimeout);  // UStimeout is set to 20000 =20ms
  if (UStime==0) {UStime =UStimeout;}    // timeout returns 0, so make it a big distance
  //Serial.print(UStime);
  if (f_cm) {return UStime/uscm;} else {return UStime/usinch;}
  }	  
//-----------------------------------------------------------     
void showUSobstacle()        // check objectRange from Ultrasonic module HC-SR04
{ Serial.print("[U");
  Serial.print(objectRange);
  Serial.print(",");
  Serial.print(SonarTooNear);
  Serial.print("|");
  Serial.print(SonarAwareAt);
  Serial.print("]");
 }
//-----------------------------------------------------------     
void readLight()  // read ambient Light levels   0 - 1023
//reading every ms for 8ms ensures that a peak (or near peak) of 100Hz flicker is captured.   
{ digitalWrite(RwhiskerLEDpin_, HIGH);             // whisker LED OFF in case it was on
  digitalWrite(LwhiskerLEDpin_, HIGH);             // whisker LED OFF in case it was on
  int readings =8;
  LeyeLight =0; ReyeLight =0;
  while(readings--)
  { ReyeLight =max(ReyeLight,analogRead(RwhiskerSensePin));  // measure light level
    LeyeLight =max(LeyeLight,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
    }
  if(!f_BeQuiet)
  { Serial.print("[a");	
    Serial.print(LeyeLight);
    Serial.print("|");
    Serial.print(ReyeLight);
    Serial.print("]");
    }
  }
//----------------------------------------------------------- 
void readWhiskers()         //16ms total delay,    see also //== ALTERNATIVE CODE for readBeard()
//reading every ms for 8ms ensures that a peak (or near peak) of 100Hz flicker is captured.   
{ LwhiskerLight =0;  RwhiskerLight =0;  LwhiskerDark =0;  RwhiskerDark =0;
  digitalWrite(RwhiskerLEDpin_, LOW);                              // whisker LED ON
  digitalWrite(LwhiskerLEDpin_, LOW);                              // whisker LED ON
  int readings =8;
  while(readings--)
  { RwhiskerLight =max(RwhiskerLight,analogRead(RwhiskerSensePin));  // measure light level
    LwhiskerLight =max(LwhiskerLight,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
    }	
  digitalWrite(RwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  readings =8;
  while(readings--)
  { RwhiskerDark =max(RwhiskerDark,analogRead(RwhiskerSensePin));  // measure light level
    LwhiskerDark =max(LwhiskerDark,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
    }	
  RwhiskerDifference =RwhiskerLight-RwhiskerDark;
  LwhiskerDifference =LwhiskerLight-LwhiskerDark;
  if(!f_BeQuiet && f_bv){ showWhiskerLightLevels(); }
  }
//----------------------------
void showWhiskerLightLevels()     //print light levels to Serial Monitor
{ //39ms 
  Serial.print("[W");
  Serial.print(LwhiskerLight);
  Serial.print(",");
  Serial.print(LwhiskerDark);
  Serial.print(",");
  Serial.print(LwhiskerLight -LwhiskerDark);
  Serial.print("|");
  Serial.print(RwhiskerLight);
  Serial.print(",");
  Serial.print(RwhiskerDark);
  Serial.print(",");
  Serial.print(RwhiskerLight -RwhiskerDark);
  Serial.print("]");
  }  
//----------------------------
void queryObstaclesLR()
// calculate difference in light value with LED on and LED off and compare to obstacleThreshold
{ if ( RwhiskerDifference > obstacleThresholdRight ) {f_obstacleRight =1;} else {f_obstacleRight =0;}
  if ( LwhiskerDifference > obstacleThresholdLeft )  {f_obstacleLeft =1;} else {f_obstacleLeft =0;}
  if(!f_BeQuiet) { showObstacles(); }
  }
//----------------------------
void showObstacles()
{ Serial.print("[o%");
  Serial.print(f_obstacleLeft);
  Serial.print(f_obstacleRight);
  Serial.print("]");
  }
//----------------------------
void readBeard()         //16ms total delay,    see also //== ALTERNATIVE CODE
{ LbeardLight =0;  RbeardLight =0;  LbeardDark =0;  RbeardDark =0;
  digitalWrite(RbeardLEDpin_, LOW);                         // beard LED ON
  digitalWrite(LbeardLEDpin_, LOW);                         // beard LED ON
  int readings =8;
  while(readings--)
  { LbeardLight =max(LbeardLight,analogRead(LbeardSensePin)); // measure light level
    RbeardLight =max(RbeardLight,analogRead(RbeardSensePin)); // measure light level
    delay(1);
    }	
  digitalWrite(RbeardLEDpin_, HIGH);                        // beard LED OFF
  digitalWrite(LbeardLEDpin_, HIGH);                        // beard LED OFF
  readings =8;
  while(readings--)
  { RbeardDark =max(RbeardDark,analogRead(RbeardSensePin));   // measure dark level  
    LbeardDark =max(LbeardDark,analogRead(LbeardSensePin));   // measure dark level
    delay(1);
    }	
  RbeardDifference =RbeardLight-RbeardDark;
  LbeardDifference =LbeardLight-LbeardDark;
  if(!f_BeQuiet && f_bv){ showBeardLightLevels(); }
  }
//----------------------------
void showBeardLightLevels()     //print light levels to Serial Monitor
{ //39ms
  Serial.print("[B");
  Serial.print(LbeardLight);
  Serial.print(",");
  Serial.print(LbeardDark);
  Serial.print(",");
  Serial.print(LbeardLight -LbeardDark);
  Serial.print("|");
  Serial.print(RbeardLight);
  Serial.print(",");
  Serial.print(RbeardDark);
  Serial.print(",");
  Serial.print(RbeardLight -RbeardDark);
  Serial.print("]");
  } 
//----------------------------
void queryEdgeLR()
// calculate difference in light value with LED on and LED off and compare to holeThreshold
{ if ( RbeardDifference < holeThresholdRight ) {f_holeRight =1;} else {f_holeRight =0;}
  if ( LbeardDifference < holeThresholdLeft )  {f_holeLeft =1;} else {f_holeLeft =0;}
  if(!f_BeQuiet) { showHoles(); }
  }
//----------------------------
void showHoles()
{ Serial.print("[h%");
  Serial.print(f_holeLeft);
  Serial.print(f_holeRight);
  Serial.print("]");
  }
//== ALTERNATIVE CODE =======================================
/*
void readBeard()         //8ms total delay - not as consistent readings
{ LbeardLight =0;	RbeardLight =0;	LbeardDark =0;	RbeardDark =0;
  int readings =8;
  while(readings--)
  { LbeardDark =max(LbeardDark,analogRead(LbeardSensePin));   // measure dark level  
    digitalWrite(LbeardLEDpin_, LOW);                         // beard LED ON
    LbeardLight =max(LbeardLight,analogRead(LbeardSensePin)); // measure light level
    digitalWrite(LbeardLEDpin_, HIGH);                        // beard LED OFF
    RbeardDark =max(RbeardDark,analogRead(RbeardSensePin));   // measure dark level
    digitalWrite(RbeardLEDpin_, LOW);                         // beard LED ON
    RbeardLight =max(RbeardLight,analogRead(RbeardSensePin)); // measure light level
    digitalWrite(RbeardLEDpin_, HIGH);                        // beard LED OFF
	  delay(1);
    }	
  RbeardDifference =RbeardLight-RbeardDark;
  LbeardDifference =LbeardLight-LbeardDark;
  showBeardLightLevels();
  }
*/
//===========================================================
