// Bambino
// TAB FILE > adjustServos45.ino

/*== CHANGED ================================================
  adjusted EEPROM.write and EEPROM.read to include BehaviourEbase so correct slot is written to.
*/

  
// based on B_PM04_Servos.ino
/*
  Seting the servos with AdjustServos45.ino
  =========================================
  This allows you to set the mid positions of both servos.
  When the servos are set to 90 degrees as in sketch BM01_ServosMid, because of 
  the way each servo is made and the horns can only be put on about every 15 
  degrees, the feet may not be exactly side by side or flat on the ground.
  Load this sketch and open the Serial Monitor.
  Adjust the Roll servo so Bambino stands up straight and doesn't lean to the left or right.
  Adjust the Pace servo so Bambino stands with both feet side by side.
  The calibrated values are stored in EEPROM for further routines.

   Set servos to mid position - rollC and paceC, to check if feet are flat and feet together      
     if not change rollC and paceC until they are
   Set the Pace length and Roll amount for walking.	 
   You can also set servos maximum and minimum safe angles for 
   Roll Right and Roll Left, and Pace Left Forward and Left Back.
   
   Use the Arduino Serial Monitor set to 'no line ending',
     type Q, ., [Esc], I, *, -, +, /, C, R, P, S (or L, H) and press ENTER
     to send Q, [dot], [Esc], I, *, -, +, /, C, R, P, S (or L, H)   
   Or use a Serial Terminal at 9600 baud. 
     [Q] or [.] or [Esc] to quit adjusting
     [I] to display instructions
     [*] to display current EEPROM values
     [-] adjust Servo position -ve
     [+] adjust Servo position +ve
     [/] to change servo
     [C] save the Roll or Pace positions as default Centre angles RollC or PaceC to EEPROM 
     [R] save the Roll amount as default to EEPROM 
     [P] save the Pace length as default to EEPROM
     [S] Stand with the servos at RollC and PaceC
    ([L] save the Low=Minimum angle to EEPROM  - future use )
    ([H] save the High=Maximum angle to EEPROM - future use )
  instructions and current settings appear on screen.
  Servo values are in units of 1 degree
  nominal servo range is 0 to 180 with centre at 90
*/	   
//===========================================================
void AdjustServos()
{ // Variables
  boolean f_rollservo =true;         // for switch case
  
  ServosInitC(); 
  printAdjustInstructions();
  printEEPROMvalues();  
  Serial.println("First the Roll servo");
  Serial.print(F("roll ="));
  Serial.println(rollat);  
  
  Ccmnd =' ';
  f_walking =true;
  
  do
  {
    switch (Ccmnd) 
    {
      case 'i':
      case 'I':
        printAdjustInstructions();
        break;        
      case '*':
        printEEPROMvalues();
        break;        
      case '/':
        f_rollservo =!f_rollservo;     // !true ->false, !false ->true
        break;        
      case '-': 
        if (f_rollservo) 
          { rollat -=1; }
         else 
          { paceat -=1; }
        break;
      case '+': 
        if (f_rollservo) 
          { rollat +=1; }
         else 
          { paceat +=1; }
        break;
      case 'h':
      case 'H': 	  
        if (f_rollservo) 
          {
          EEPROM.write(BehaviourEbase +Es25rollmax,rollat); 
          Serial.print(F(" rollmax =")); Serial.println(rollat);}
         else 
          {
          EEPROM.write(BehaviourEbase +Es23pacemax,paceat); 
          Serial.print(F(" pacemax =")); Serial.println(paceat);}
        break;
      case 'l': 
      case 'L': 
        if (f_rollservo) 
          {
          EEPROM.write(BehaviourEbase +Es26rollmin,rollat); 
          Serial.print(F(" rollmin =")); Serial.println(rollat);}
         else 
          {
          EEPROM.write(BehaviourEbase +Es24pacemin,paceat); 
          Serial.print(F(" pacemin ="));  Serial.println(paceat);}
        break;
      case 'c': 
      case 'C': 
        if (f_rollservo) 
          {
          EEPROM.write(BehaviourEbase +Es30rollC,rollat); 
          Serial.print(F(" rollCentre =")); Serial.println(rollat);
          rollC =rollat;
          }
         else 
          {
          EEPROM.write(BehaviourEbase +Es29paceC,paceat); 
          Serial.print(F(" paceCentre =")); Serial.println(paceat);
          paceC =paceat;
          }
        break;
      case 'r': 
      case 'R': 
        rollby =abs(rollat -rollC); 
        EEPROM.write(BehaviourEbase +Es28rollby,rollby); 
        Serial.print(F(" RollBy =")); Serial.println(rollby); 
        break;
      case 'p':
      case 'P':
        paceby =abs(paceat -paceC);  
        EEPROM.write(BehaviourEbase +Es27paceby,paceby); 
        Serial.print(F(" PaceBy =")); Serial.println(paceby);
        break;
      case 's': 
      case 'S': 
        Serial.println(F("StandC"));
        ServosInitC();
        break;

    }    // end switch
      
    
    if (f_rollservo) 
      { Serial.print(F("roll =")); Serial.println(rollat); }
     else 
      { Serial.print(F("pace =")); Serial.println(paceat); }

  
    ServoPulse(servoRollPin,rollat);            // tell servo to go to position rollat
    ServoPulse(servoPacePin,paceat);            // tell servo to go to position paceat
    delay(Servoframe);                          // delay for servo frame 
    GetCcmnd();
  } while (Ccmnd!='.' &&  Ccmnd!=Esc && Ccmnd!='Q' &&Ccmnd!='q') ; 
} 
//===========================================================
void printEEPROMvalues()
{
  Serial.println(F("Current values in EEPROM"));
  Serial.print(F("  RollCentre ="));    Serial.println(EEPROM.read(BehaviourEbase +Es30rollC));
  Serial.print(F("  PaceCentre ="));    Serial.println(EEPROM.read(BehaviourEbase +Es29paceC));
  Serial.print(F("  RollBy     ="));    Serial.println(EEPROM.read(BehaviourEbase +Es28rollby));  
  Serial.print(F("  PaceBy     ="));    Serial.println(EEPROM.read(BehaviourEbase +Es27paceby));
  Serial.print(F("  Rollmin    ="));    Serial.println(EEPROM.read(BehaviourEbase +Es26rollmin));
  Serial.print(F("  Rollmax    ="));    Serial.println(EEPROM.read(BehaviourEbase +Es25rollmax));
  Serial.print(F("  Pacemin    ="));    Serial.println(EEPROM.read(BehaviourEbase +Es24pacemin));
  Serial.print(F("  Pacemax    ="));    Serial.println(EEPROM.read(BehaviourEbase +Es23pacemax));
}

void printAdjustInstructions()
{ Serial.println();
  Serial.println(F("Determine and save the maximum and minimum safe values"));
  Serial.println(F("Determine and save the Servo centre position"));
  Serial.println(F("Determine and save the default Pace and Roll values for walking"));
  Serial.println(F("Servo values are nominal 1 degree, 0 - 180, centre at 90"));
  Serial.println(F("hit [Q] or [.] or [Esc] to quit adjusting"));
  Serial.println(F("hit [I] to display instructions"));
  Serial.println(F("hit [*] to display current EEPROM values"));
  Serial.println(F("hit [-] to adjust the value and move the Servo"));
  Serial.println(F("hit [+] to adjust the value and move the Servo"));
  Serial.println(F("hit [/] to toggle between servos"));  
  Serial.println(F("hit [C] to set the centre position value"));
  Serial.println(F("hit [P] to set the default Pace length"));
  Serial.println(F("hit [R] to set the default Roll amount"));
  Serial.println(F("hit [S] to get Bambino to Stand at rollC, paceC"));
  Serial.println(F("hit [L] to set the minimum safe value - future use"));
  Serial.println(F("hit [H] to set the maximum safe value - future use"));
  Serial.println(F("If using the Serial Monitor -"));
  Serial.println(F("  hit [Enter] or click 'Send'"));
  Serial.println(F("  to send the character"));
}  
  
//===========================================================
 
