//BA52_IRout.ino

char thisrobot[] ="Bambino";
char thisprog[]  ="BA52_IRout"; 

//PM Sketch uses 30496 bytes (96%) of program storage space. Maximum is 30720 bytes.
//PM Global variables use 1131 bytes (55%) of dynamic memory, leaving 919 bytes for local variables. Maximum is 2048 bytes.
//M2 Sketch uses 30520 bytes (89%) of program storage space. Maximum is 32256 bytes.
//M2 Global variables use 1211 bytes (55%) of dynamic memory, leaving 919 bytes for local variables. Maximum is 2048 bytes.

//set Serial terminal to 9600 baud
//== #define ================================================
//#define BambinoPM         //ProMini 328,5v
#define BambinoMM         //Uno with no Whisker sensors, Malcolm Margolis PCB
//#define BambinoM2         //Uno, Malcolm Margolis PCBv2 

//== Introducing ==========================================
// command 'T' to tell an IR cmnd
// command 'n' to nap for tenths seconds
//== History ==============================================
//52   2022-02-18 - 'n' nap, 'T' IRout()
//51   2022-02-05 - Quit in WanderSteps()
//51   2022-02-05 - chatter
//51   2022-02-05 - *r *w for future use of higher processor
//51   2022-02-05 - 'a' - actions
//50.2 2022-02-04 - Bugs
//50.1 2022-02-04 - Bugs
//== Notes ================================================
// - The Arduino Serial Monitor can't send [Esc]. 
//   If another Terminal is used which can send [Esc] then at GetCcmnd() this will be printed,
//   which may print as a symbol or the Terminal may change it to "."
//== Bugs ==================================================
//GetAct() didn't work with IR, corrected
//recording with IR didn't beep, corrected
//== NEW ====================================================
// 'T' - send IR cmnd
// 'n' nap tenths seconds
//void IRout()
//void translate2IR() {   
//void IR_on(byte cycles, boolean state ) {
//== CHANGED ================================================
//== INCLUDES ===============================================
#include "pins20.h"
#include "varsCons50.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
  
//== VARIABLES and CONSTANTS ================================
//see also TAB FILE > varsCons50.h
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play current Act
boolean f_reacting =false;    //in a ReAct - set in behaviour44.ino if a React is initiated.
boolean f_inDo =false;
boolean f_runDefault =false;  //run Default when nothing else to do
boolean f_skipmind =false;    //so commands in a mind which are skipped do not print in GetCmnd
boolean f_IRcmnd =0;          //Ccmnd is by IR                          //50
const byte NL =10;            //NewLine value
const byte LF =10;            //LineFeed value                                  
const byte CR =13;            //Carriage Return value
const byte Esc =27;           //Esc key value
char Act;                     //current Act as '1' - '8'
byte actIndex =0;             //current Act number, set to default           
int EEPROMindex =-1;          //index to moves in an Act in EEPROM, -1 => nothing to continue with #>
int EEadress;                 //used in *r and *w 
char hashgroup =' ';          // current hash group  
int hashhashAt;               //copy of EEPROMindex at hashhashSTOP 
int ReactAt;                  //copy of EEPROMindex at React
int ActEnd;                   //end address of current recording Act in EEPROM
int DoAt;                     //return address in a Do
int moves =0;                 //current move
char Ccmnd =0;                //current Character command, default null command
char oldCcmnd;                                                       //50
byte epage =1;                //the Act page for the current Act, default page =1 for  Acts 1-8 and page =2 for 01-08
char Behaviour;               //read from EEPROM in init, current behaviour 1-8 in epage 0
byte BehaviourEbase =224;     //base address of current behaviour 0,32,64,96,128,160,192,(224)
byte slotbase;                //working slotbase
byte behaveNum;               //BeHave address in Behaviour slot 
byte behaveValue;
String numString ="";         //temporary string in loadBehaviour(), getnum()
byte mchance;                 //chance of executing Mind cmnds   
/*=================================================================
TVR010_PicAxe_IR_controller:
'---------------------------
'Before use, the transmitter must be programmed with the ‘Sony’ transmit code.
'1. Insert 2 AAA size batteries, preferably alkaline.
'2. Press ‘S’ and ‘B’ at the same time. S is in the centre of the arrows.
'  The top left red LED should light.
'3. Press ‘0’. The LED should flash.
'4. Press ‘1’. The LED should flash.
'5. Press ‘3’. The LED should go out.
'6. Press the red power button (top right).
'-------------------------------------------
'IRin cmnds using PicAxe controller
'DO NOT PRESS OTHER KEYS
'ie [A] [B] [C] [D] [E] [F] [G]
'They change the Mode and [B] has to be pressed to change back.
'[square] [triangle] [()] [L] [X] [backwards F]  have no effect
*/
// NOT ALL COMMANDS IMPLEMENTED
const byte keyPOWER  = 21;    //Sleep

const byte keyUP     = 16;    //[Fd] Step Forward        [9][Fd]=set vitality Behaviour to faster
const byte keyDOWN   = 17;    //[Bk] Step Backward       [9][Bk]=set vitality Behaviour to slower
const byte keyRIGHT  = 18;    //[Rt] Step Right
const byte keyLEFT   = 19;    //[Lt] Step Left

const byte keyBAR    = 96;    //[f|] learn an Act, next [key] selects an Act
const byte keyTENT   = 54;    //[f^] next [key] selects a Subroutine-Act
const byte keyVCROSS = 37;    //[f+] Modify Mode {when routine included}
                              //     [+] set USAwareAt in DUSawareAt
                              //     [-] set UStoonear in DUStoonear
                              //    {[Fd] alter PaceCentre in DpaceC}
                              //    {[Bk] alter PaceCentre in DpaceC}
                              //    {[Rt] alter RollCentre in DrollC}
                              //    {[Lt] alter RollCentre in DrollC}
                              //    {     [-][+]  to adjust         }
                              //     [keyPOWER] to quit
const byte keyXCROSS = 20;    //[fX] transfer EEPROM to PC for including in programs
                          //    -- Subroutine-Act and Commands --
const byte key1      =  0;    //[1] [f^][1]=>RockADo      [9][1]=set speed slow
const byte key2      =  1;    //[2]                       [9][2]=set speed medium
const byte key3      =  2;    //[3]                       [9][3]=set speed fast
const byte key4      =  3;    //[4]                       [9][4]=set US behaviour to timid
const byte key5      =  4;    //[5]                       [9][5]=set US behaviour to normal
const byte key6      =  5;    //[6]                       [9][6]=set US behaviour to bold
const byte key7      =  6;    //[7]                       [9][7]=disable Whiskers test
const byte key8      =  7;    //[8]                       [9][8]=disable Ultrasonic test
const byte key9      =  8;    //[9] set Behaviour modes   [9][9]=set US behaviour to User
const byte keyMINUS  = 98;    //[-] enable & test Whisker sensors and re-act, R_Whiskers:
const byte key0      =  9;    //[0] WANDER                [9][0]=set vitality Behaviour to normal
const byte keyPLUS   = 11;    //[+] enable & test Ping sensor and re-act, R_Ping:

//== SETUP ==================================================
void setup() 
{ 
  f_BeQuiet =0;                    // send status messages
  f_MsgForMe =1;                   // listen to messages   
  f_hashhashSTOP =0;               
  
  Serial.begin(9600);      // to Serial Monitor 
#ifndef BambinoMM
  pinMode(IRoutpin,OUTPUT);
#endif  
  pinMode(voicePin,OUTPUT);
  pinMode(LbeardLEDpin_, OUTPUT);  
  pinMode(RbeardLEDpin_, OUTPUT);  
  pinMode(LwhiskerLEDpin_, OUTPUT);
  pinMode(RwhiskerLEDpin_, OUTPUT);
  digitalWrite(LbeardLEDpin_, HIGH);  // beard LED OFF
  digitalWrite(RbeardLEDpin_, HIGH);  // beard LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  
  if (EEPROM.read(E8f_walking)>1) {initialiseEEPROM();}   // in case it is not initialised
  
  //find current current behaviour in EEPROM and copy to RAM
  for (BehaviourEbase =0; BehaviourEbase < 224; BehaviourEbase +=32 )  //test first 7 slots 
    {if (EEPROM.read(BehaviourEbase) =='@')  break; }  
    // if @ not found BehaviourEbase is incremented to 224
    //if '@' not found at 8 make 8 the default 
  if (EEPROM.read(BehaviourEbase) !='@') { EEPROM.write(BehaviourEbase,'@'); }
  
  Behaviour =BehaviourEbase /32 +'1';  //'1' - '8' 
  
  getMind();

  Serial.print(thisrobot); // so we know which robot the program is for
  Serial.print(" ");
  Serial.print(thisprog);  // so we know which program is running
  Serial.print(" ");
  Serial.print(thisrobot); // so we know which robot the program is for
  Serial.print(" ");
  Serial.print(rgroup);    // so we know which group it is in
  Serial.println(rname);   // so we know which robot this is
    
  //now configure Bambino
  f_forwards =1;    // set default

  pinMode(servoRollPin, OUTPUT);    // sets the digital pin as output
  pinMode(servoPacePin, OUTPUT);    // sets the digital pin as output
  digitalWrite(servoRollPin, LOW);   // initialise pin
  digitalWrite(servoPacePin, LOW);   // initialise pin

  if (f_walking) { Serial.println(" walking ON"); } else { Serial.println(" walking OFF"); }

  ServosInitC();  //initialises rollat to rollC and paceat to paceC
  Srollat =rollat;
  Spaceat =paceat; 
}
//== LOOP ===================================================
void loop()   {

  f_recordC =true;     // record Command, default =true
  
#ifdef BambinoMM 
  redLEDdim();         //indicate ready for command except    
  if (f_recording)  { redLEDon(); }
  if (f_play)       { redLEDoff(); }
#else
  if (f_recording)  { redLEDon(); } else { redLEDoff(); }
  if (!f_reacting)                    // don't change if in a ReAct
    { if (f_play)
        { whiskerLEDsOff(); }
        else
        { whiskerLEDsDim(); }      //indicate ready for command
    }
#endif

  if (f_doDefault)   // true on the first loop and #< only
    { f_doDefault =false;                // don't do it again
      Ccmnd =autoAct;
      if(Ccmnd=='0') { f_runDefault =false;  Ccmnd ='.'; }     // '0' -> no autorun, can't run it so do stop 
    }   
   else
    // if f_hashhashSTOP skip GetCcmnd() and process Ccmnd '.' from ##
    { if (!f_hashhashSTOP)
        { if (f_bAutoSense && !f_reacting  && Ccmnd!='0')   //don't test if in a React - no return stack 
              //if just got '0' for Acts0 then Reacting looses epage before EEPROMindex is set - 2021-04-07
            { if (f_bG) { BeardReact(); }
              if (f_bW) { WhiskerReact(); }
              if (f_bU) { USreact(); }
            }
          if (!f_play && !f_reacting && !f_BeQuiet)  { Serial.print(">"); }    //ready prompt
          oldCcmnd =Ccmnd;
          GetCcmnd();    //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
        }  
    }

IRcmnds:
// PicAxe Remote Control TVR010 
// other buttons don't send codes and/or change controller mode
//                        [power]21
//               [^]16
//          [<]19     [>]18  
//               [v]17
// [bar]96 [tent]54 [vcross]37 [xcross]20      
//       [1]0     [2]1       [3]2
//       [4]3     [5]4       [6]5
//       [7]6     [8]7       [9]8
//       [-]98    [0]9       [+]11
// [square]54  [triangle]50  [circle]55 
//    [L]22       [X]56        [F]57

  if (!f_IRcmnd) { goto DoCommand; }                                                     //50          
  switch (Ccmnd)                                                                         //50 
  { case '.':  if (oldCcmnd=='.') { Ccmnd ='H'; f_walking=1; }  break;
    case '9':  NewCcmnd();     // don't get a repeat '9'
               switch (Ccmnd)
               { case '1':    rollspeed =1; pacespeed =1;  break;  //slow
                 case '2':    rollspeed =2; pacespeed =3;  break;  //medium
                 case '3':    rollspeed =3; pacespeed =6;  break;  //fast
                 case '4':    SonarTooNear =15; SonarAwareAt =30;  break;  //timid
                 case '5':    SonarTooNear =10; SonarAwareAt =20;  break;  //normal
                 case '6':    SonarTooNear =5;  SonarAwareAt =10;  break;  //bold
                 case '7':    // (Bambino) - done by -
                 case '8':    // (Bambino) - done by +
                 case 'F':    Servoframe -=3; constrain(Servoframe,10,40);  break;  //faster
                 case '0':    Servoframe =normalSframe;  break;  //normal
                 case 'B':    Servoframe +=3; constrain(Servoframe,10,40);  break;  //slower
                 case '-':    f_bG =0;  break;              //no beard
                 case '+':    f_bW =0;  f_bU =0;  break;    //no whiskers, no Ultrasonics
                }
                Ccmnd =0;      break;  //no action in DoCommand:
   } 
  waitNoIR(); 

DoCommand:
  switch (Ccmnd) 
  { case  0:    break;   //Ccmnd==0 if IR cmnd 8, so no action      
    case LF:    f_recordC =false;  break;     //linefeed
    case CR:    f_recordC =false;  break;     //carriage return
    case ' ':   break;     //strip space
    case ',':   break;     //strip comma
    case 'Q':   // fall through
    case Esc:   f_reacting =false; f_inDo =false; DoAt =0;  //fall through to '.'
    case '.':   if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }     //terminated string
                f_recording =false;
                if (f_reacting)    // end of ReAct
                  { f_reacting =false; 
                    EEPROMindex =ReactAt; beardLEDsOff(); 
                    break;
                  }
                if (DoAt !=0) 
                  { Serial.print("("); Serial.print(DoAt); Serial.print(")"); 
                    EEPROMindex =DoAt; DoAt =0; f_inDo =true;
                    break;
                  }                               
                f_play =false;        // end of Act  
                if (!f_hashhashSTOP) { EEPROMindex =-1; };   // so there is nothing to continue with #>
                f_hashhashSTOP =false;
                if(!f_BeQuiet){ Serial.println(); }
                if (f_runDefault) { f_doDefault =true; }
                break;
    case '(':   f_inDo =true;   Serial.print("="); Serial.println(EEPROMindex-1); break;
                            //    Serial.print((EEPROMindex-1)/256 -1);
                            //    Serial.print(',');
                            //    Serial.print(((EEPROMindex-1)%256)/32+1);
                            //    Serial.print(',');
//                                Serial.print((EEPROMindex-1)%32);
                            //    break;
    case ')':   f_inDo =false;  Serial.print("="); Serial.println(EEPROMindex-1); break;
                         //       Serial.print((EEPROMindex-1)/256 -1);
                          //      Serial.print(',');
                           //     Serial.print(((EEPROMindex-1)%256)/32+1);
                            //    Serial.print(',');
                             //   Serial.print((((EEPROMindex-1)%256)/32)%32);
                              //  break;
    case 'A':	  // if recording record 'A' otherwise do beAlive() until 'Q' or reset
                if (!f_recording) { beAlive(); }  break;
    case 'X':	  // if recording record 'X' otherwise do Xplore() until 'Q' or reset
                if (!f_recording) { Xplore(); }  break; 
    case 'E':	  // if recording record 'E' otherwise do Experiment() until 'Q' or reset
                if (!f_recording) { Experiment(); }  break; 
    case '!':   epage =3;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '0':   epage =2;  break;  //reset to 1 at the end of setEEPROMindexes(char Act)
    case '1':   // fall through
    case '2':   // fall through
    case '3':   // fall through
    case '4':   // fall through
    case '5':   // fall through
    case '6':   // fall through
    case '7':   // fall through
    case '8':   if (!f_recording) 
                  { if (f_inDo && DoAt==0)
                    { DoAt =EEPROMindex; Serial.print("(@"); Serial.print(DoAt); Serial.print(")"); 
                      f_inDo =false;        //because we are jumping to another Act
                    }  
                    f_play =true; Act =Ccmnd;  setEEPROMindexes(Act); 
                  } break;
    case '9':   f_hashhashSTOP =false;  f_doDefault =true;  break;
    case 'F':   forward();      break;        
    case 'B':   backward();     break;        
    case 'R':   turnRight();    break;        
    case 'L':   turnLeft();     break;        
    case 'H':   halt();         break; 
    case 'V':   whee();         break;   //Spkr and LED on different pins           
    case 'S':   f_recordC =false;
                toggle_f_walking(); 
                if(!f_BeQuiet)
                  {if (f_walking) {Serial.println(" Walking ON");} else {Serial.println(" Walking OFF");}}
                break;    
    case '-':   f_bG =1;            BeardReact();                break; 
    case '+':   f_bW =1;  f_bU =1;  WhiskerReact();  USreact();  break; 
    case 'U':   USreact();      break; 
    case 'W':   WhiskerReact(); break; 
    case 'G':   BeardReact();   break;
    
actions:
#ifndef BambinoMM
    case 'T':   if (f_recording) { RecordCcmnd(); }    //so 'T' is recorded
                GetCcmnd(); IRout();                   //get parameter and Tell it
                break;
#endif                
    case 'a':   if (f_recording) { RecordCcmnd(); }    //so 'a' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd) 
                { case '1':  bip();   break;           //also here for IR
                  case '2':  whee();  break;           //also here for IR
                  case '3':  whoa();  break;           //also here for IR
                  case '4':  chatter();  break;        //also here for IR
                  case '5':  //drop through
                  case 'r':  if (f_recording) { RecordCcmnd(); }    //so 'r' is recorded
                             GetCcmnd();                            //get parameter
                             if(Ccmnd<'1' || Ccmnd >'9'){Ccmnd ='1';}
                             rock(Ccmnd -'0');     break;
                  case '6':  //drop through           
                  case 's':  if (f_recording) { RecordCcmnd(); }    //so 's' is recorded
                             GetCcmnd();                            //get parameter
                             if(Ccmnd<'1' || Ccmnd >'9'){Ccmnd ='1';}
                             shuffle(Ccmnd -'0');  break;
                }    // end switch
                break;      
voice:    
    case 'v':   if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd) 
                { case 'b':  bip();   break;
                  case 'e':  whee();  break;
                  case 'o':  whoa();  break;
                  case 'c':  chatter();  break;
                  default:   bip();   Ccmnd ='b';  break;
                }    // end switch
                break;      
wait:
    case 'n':   if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded
                GetCcmnd();                            //get parameter
                makeCcmndDigit();                      //ensure '0'-'9'
                delay((Ccmnd -'0') *100);              //delay 1/10s  secs
                break;
    
    case 'w':   if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded
                GetCcmnd();                            //get parameter
                makeCcmndDigit();                      //ensure '0'-'9'
                delay((Ccmnd -'0') *1000);             //delay seconds
                break;
speed:                
    case 's':   if (f_recording) { RecordCcmnd(); }    //so 's' is recorded
                GetCcmnd();                            //get parameter
                if (Ccmnd=='u') { speedup();  break; }     // and if recording record 'u' 
                if (Ccmnd=='d') { speeddown();  break; }   // and if recording record 'd' 
                if (Ccmnd=='m') { rollspeedmax =rollspeed; pacespeedmax =pacespeed; break; }
                if (Ccmnd=='r') 
                  { if (f_recording) { RecordCcmnd(); }  //so 'r' is recorded
                    GetCcmnd();                          //get parameter
                    if (Ccmnd=='m') { rollspeedmax =rollspeed; break; }
                    //else                 
                    makeCcmndDigit();                    //ensure '0'-'9'    
                    rollspeed =max(1,Ccmnd -'0');        //ensure 1 - 9    
                    break;
                  }
                if (Ccmnd=='p') 
                  { if (f_recording) { RecordCcmnd(); }  //so 'p' is recorded
                    GetCcmnd();                          //get parameter
                    if (Ccmnd=='m') { pacespeedmax =pacespeed; break; }
                    //else                 
                    makeCcmndDigit();                    //ensure '0'-'9'    
                    pacespeed =max(1,Ccmnd -'0');        //ensure 1 - 9    
                    break;
                  }
                //else                 
                makeCcmndDigit();                      //ensure '0'-'9'    
                rollspeed =max(1,Ccmnd -'0');          // hangs if 0 
                pacespeed =max(1,Ccmnd -'0');          // hangs if 0       	
                break;
turn:                
    case 't':   if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                { case 'a':  turnAway();    break;
                  case 'm':  turnMemory();  break;
                  case 'o':  turnOther();   break;
                  case 'l': turnLight();   break;
                  case 'd': turnDark();    break;
                  case '@': if (f_recording) { RecordCcmnd(); }    //so '@' is recorded
                            GetCcmnd();                            //get parameter
                            switch (Ccmnd)
                            { case 'l':  turnAtLight();  break;
                              case 'd':  turnAtDark();   break;
                              default:   badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                                         Ccmnd ='?';     //so if recording then '?' is recorded
                                         break;          //don't do anything 
                            }    // end switch
                  default:  turnMemory();   Ccmnd ='m';  break;
                }     // end switch case t
                break;
    case 'm':   if (f_recording) { RecordCcmnd(); }    //so 'm' is recorded
                GetCcmnd();                            //get parameter     
                mchance =random(0,10);
                //Serial.println(mchance);
                if (!f_recording && Ccmnd -'0' < mchance)        // don't skip if recording
                { f_skipmind =true; 
                  while(Ccmnd!='}') { GetCcmnd(); } 
                  f_skipmind =false; printCcmnd();     //so '}' is printed after skipping
                  }
                break;
    case '}':   break;  //end of a mind, ignore it

//---behaves----------------------------------------                
    case 'b':   if (f_recording) { RecordCcmnd(); }    //so 'b' is recorded
                GetCcmnd();                            //get parameter
                switch (Ccmnd)
                {
  b_behaviour:    case '0': if (f_recording) { RecordCcmnd(); }      //so Ccmnd is recorded
                            getMind();  break;
                  case '1': // fall through
                  case '2': // fall through
                  case '3': // fall through
                  case '4': // fall through
                  case '5': // fall through
                  case '6': // fall through
                  case '7': // fall through
                  case '8': if (f_recording) { RecordCcmnd(); }      //so Ccmnd is recorded
                            Behaviour =Ccmnd;
                            EEPROM.write(0 +BehaviourEbase,'.');     //deselect old behaviour 
                            BehaviourEbase =(Behaviour -'1') *32;    //new behaviour 
                            EEPROM.write(0 +BehaviourEbase,'@');     //new behaviour is current
                            getMind();
                            break;
  b_save:         case 's': if (f_recording) { RecordCcmnd(); }    //so 's' is recorded
                            GetCcmnd();                            //get parameter, if cmnd='0' then save to current behaviour  
                            if(Ccmnd<'0' || Ccmnd>'8') { Ccmnd ='?'; break; }
							if (f_recording) { RecordCcmnd(); }          //so Ccmnd is recorded
                            if(Ccmnd=='0') {slotbase =BehaviourEbase;} else {slotbase =(Ccmnd -'1') *32;}
                            saveMind();                 // save current behaviour to a behaviour slot
                            break;
  b_load:         case 'l': if (f_recording) 
                              { Ccmnd ='.'; RecordCcmnd();  //so '.' is recorded
                                f_recording =false;
                                break;
                               }
                            if (!f_recording) {loadBehaviour();}
                            break;	
  b_hash:         case '#': if (f_recording) { RecordCcmnd(); }    //so '#' is recorded
                            GetCcmnd();                     //get parameter
                            if (Ccmnd>='1' && Ccmnd<='9') 
                              { rgroup =Ccmnd;  break; }
                            if (Ccmnd>='A' && Ccmnd<='z') 
                              { rname =Ccmnd;  break; }
                            badkey();
                            break;
  b_autoRun:      case 'a': if (f_recording) { RecordCcmnd(); }    //so 'a' is recorded
                            GetCcmnd();                            //get parameter
                            if(Ccmnd<'0' || Ccmnd>'9') { Ccmnd ='0'; break; }
                            if (Ccmnd=='9') {f_runDefault =true; break;}          // ## sets f_runDefault false
                            autoAct =Ccmnd;
                            break;
  b_quiet:        case 'q': f_BeQuiet =1;  break; 
  b_walking:      case 'w': if (f_recording) { RecordCcmnd(); }    //so 'w' is recorded 
                            GetCcmnd();                            //get parameter
                            if (Ccmnd=='0') { disable_walking();  break; }   // and if recording record '0' 
                            if (Ccmnd=='1') { enable_walking();   break; }   // and if recording record '1' 
                            if (Ccmnd=='f') { f_forwards =true;   break; }   // and if recording record 'f' 
                            if (Ccmnd=='b') { f_forwards =false;  break; }   // and if recording record 'b' 
                            Ccmnd ='?';   // bad char, no action and if recording record '?' 
                            break;                                 // if recording Ccmnd will be recorded                                                       
  b_pace:         case 'p': if (f_recording) { RecordCcmnd(); }    //so 'p' is recorded
                            GetCcmnd();                            //get parameter
                            switch (Ccmnd)
    b_paceCplus:            { case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        paceC =min(EEPROM.read(BehaviourEbase +Es29paceC) +maxpace,paceC +(Ccmnd -'0'));  //restrict to maxpace  
                                        break;
    b_paceCminus:             case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        paceC =max(EEPROM.read(BehaviourEbase +Es29paceC) -maxpace,paceC -(Ccmnd -'0'));  //restrict to maxpace  
                                        break;                                        
    b_paceby:                 default:  if (Ccmnd<'0' || Ccmnd>'9') 
                                          { badkey();  if(!f_BeQuiet){Serial.println("bad key"); }
                                            Ccmnd ='?';     //so if recording then '?' is recorded
                                            break;
                                           }
                                        paceby =(Ccmnd -'0') *pacebyScale;
                            }            
                            break;
  b_turnby:       case 't': if (f_recording) { RecordCcmnd(); }    //so 't' is recorded
                            GetCcmnd();                            //get parameter
                            makeCcmndDigit();                      //ensure '0'-'9'
                            turnby =(Ccmnd -'0') *turnbyScale;
                            break;
  b_roll:         case 'r':  if (f_recording) { RecordCcmnd(); }    //so 'r' is recorded
                            GetCcmnd();                            //get parameter
                            switch (Ccmnd)                            
    b_rollCplus:            { case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        rollC =min(EEPROM.read(BehaviourEbase +Es30rollC) +maxroll,rollC +(Ccmnd -'0'));  //restrict to maxroll
                                        break;
    b_rollCminus:             case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        rollC =max(EEPROM.read(BehaviourEbase +Es30rollC) -maxroll,rollC -(Ccmnd -'0'));  //restrict to maxroll
                                        break;
    b_rollby:                 default:  if (Ccmnd<'0' || Ccmnd>'9') 
                                          { badkey();  if(!f_BeQuiet){Serial.println("bad key"); }
                                            Ccmnd ='?';     //so if recording then '?' is recorded
                                            break;
                                           }
                                        rollby =(Ccmnd -'0') *rollbyScale;
                            }            
                            break;
  b_sonar:        case 'u': if (f_recording) { RecordCcmnd(); }    //so 'u' is recorded
                            GetCcmnd();                             //get parameter
                            switch (Ccmnd)
    b_sonarRange:           { case 'm': // don't record 'm'
                                        do
                                        { Serial.println("press [+] for AwareAt, [-] for TooNear, [.] or [Esc] to Quit");
                                          GetCcmnd();                            //get parameter
                                          objectRange =dist_HC_SR04(); if(f_cm) {objectRange /=5;} else {objectRange /=2;}
                                          if (Ccmnd=='+') 
                                            { SonarAwareAt =objectRange; 
                                              if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                              Ccmnd ='0' +SonarAwareAt; if (f_recording) { RecordCcmnd(); }    //so 'SonarAwareAt' is recorded
                                              }
                                          if (Ccmnd=='-')
                                            { SonarTooNear =objectRange;
                                              if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                              Ccmnd ='0' +SonarTooNear; if (f_recording) { RecordCcmnd(); }    //so 'SonarTooNear' is recorded
                                              }
                                        } while (Ccmnd!='.' &&  Ccmnd!='Q' && Ccmnd!=Esc);
                                        break;
    b_sonarAwareAt:           case '+': if (f_recording) { RecordCcmnd(); }    //so '+' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarAwareAt =Ccmnd -'0';
                                        break;
    b_sonarTooNear:           case '-': if (f_recording) { RecordCcmnd(); }    //so '-' is recorded
                                        GetCcmnd();                            //get parameter
                                        makeCcmndDigit();                      //ensure '0'-'9'
                                        if (f_recording) { RecordCcmnd(); }    //so Ccmnd is recorded
                                        SonarTooNear =Ccmnd -'0';
                                        break;
    b_sonarUnits:             case 'c': if (f_recording) { RecordCcmnd(); }    //so 'c' is recorded
                                        GetCcmnd();                            //get parameter
                                        if((Ccmnd -'0') ==1) { f_cm =true; } else { f_cm =false; }
                                        break;
    b_sonarSave:              case 's': EEPROM.write(BehaviourEbase +Es19SonarAwareAt,SonarAwareAt);
                                        EEPROM.write(BehaviourEbase +Es18SonarTooNear,SonarTooNear);
                                        break;
                            }     // end switch
                            break;
  b_GroundReact:  case 'G': if (f_recording) { RecordCcmnd(); }    //so 'G' is recorded
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='0') { f_bG =0; } 
                            else 
                            { Ccmnd ='1'; f_bG =1; }               //default '1'
                            break;
  b_WhiskerReact: case 'W': if (f_recording) { RecordCcmnd(); }    //so 'W' is recorded
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='0') { f_bW =0; } 
                            else 
                            { Ccmnd ='1'; f_bW =1; }               //default '1'
                            break;
  b_SonarReact:   case 'U': if (f_recording) { RecordCcmnd(); }    //so 'U' is recorded
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='0') { f_bU =0; } 
                            else 
                            { Ccmnd ='1'; f_bU =1; }               //default '1'
                            break;
  b_AutoSense:    case '$': if (f_recording) { RecordCcmnd(); }    //so '$' is recorded
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='1') { f_bAutoSense =1; }
                            else 
                            { Ccmnd ='0'; f_bAutoSense =0; }       //default '0'   
                            break;
  b_verbose:      case 'v': if (f_recording) { RecordCcmnd(); }    //so 'v' is recorded          
                            GetCcmnd();                            //get parameter           
                            if (Ccmnd=='1') { f_bv =1; }
                            else 
                            { Ccmnd ='0'; f_bv =0; }               //default '0'   
                            break;
  b_ServoAdjust:  case '@': if( f_BeQuiet==1 ) { break; }     // don't adjust if being quiet
                            AdjustServos();  break;           
  b_default:      default:  badkey();  if(!f_BeQuiet){Serial.println("bad key");}
                            Ccmnd ='?';     //so if recording then '?' is recorded
                            break;          //don't do anything
                }     // end switch
                break;                               
//------------end case 'b'--------------------------

SYSTEM:  
    case 'r':   f_recordC =false;
                redLEDon();
                if (!f_recording) 
                  { getAct();  if(Ccmnd=='.') { break; }  
                  setEEPROMindexes(Act);  f_recording =true; 
                  }
                break;
    case 'c':   f_recordC =false;  if (!f_recording) { copyAct(); }  break;	
    case 'l':   f_recordC =false;  if (!f_recording) { loadAct(); }  break;	
    case '?':   f_recordC =false;
                if (!f_recording) 
                  { GetCcmnd();                           //get parameter
                    if( f_BeQuiet==1 ) { break; }         // don't print if being quiet
                    switch (Ccmnd)
                    { case 'h':  printHelp();  break;     // print instructions
                      case '1':  // fall through
                      case '2':  // fall through
                      case '3':  // fall through
                      case '4':  // fall through
                      case '5':  // fall through
                      case '6':  // fall through
                      case '7':  // fall through
                      case '8':  Act =Ccmnd;  printAct();  break;
                      case 'p':  PrintAllActs();  break;
                      case '?':  PrintAllActs();  break;
                      case 'd':  DumpActs();  break;
                      case 'b':  printBehaviour();  break;
                      case 'm':  printMind();  break;      // - all Acts, Behaviour
                      case '!':  PrintReActs(); break;
                      case 'B':  PrintBehaviours(); break;
                      case '$':  readUSdistance();  break;
                      default:   badkey();  if(!!f_BeQuiet){Serial.println("?");}  break;
                    }    // end switch               
                  }    // end if 
                break;
    case '#':   f_recordC =false;
                f_MsgForMe =1;              // so next GetCcmnd() works
                f_hashcmnd =true;           // don't get a Play command here
                GetCcmnd();                 // get parameter
                f_hashcmnd =false;
                switch (Ccmnd)
                { case '0': f_MsgForMe =1;  f_BeQuiet =1;  hashgroup =rgroup;  break;  //everybody listen
                  case '#': f_MsgForMe =1;  f_BeQuiet =0;                  //everybody stop
                            f_hashwalking =f_walking;
                            disable_walking();  Ccmnd ='.';  hashhashAt =EEPROMindex;
                            f_hashhashSTOP =true;  // so '.' doesn't clear EEPROMindex
                            f_hashrunDefault =f_runDefault;  f_runDefault =false;
                            f_hashbAutoSense =f_bAutoSense;  f_bAutoSense =false;
                            break;
                  case '<': f_doDefault =true;  f_hashhashSTOP =false;  
                            f_runDefault =f_hashrunDefault;
                            f_bAutoSense =f_hashbAutoSense;
                            enable_walking();
                            break;
                  case '>': f_runDefault =f_hashrunDefault;
                            f_bAutoSense =f_hashbAutoSense;
                            if (hashhashAt!=-1)                //something to continue
                              { f_play =true;  EEPROMindex =hashhashAt; }
                            f_hashhashSTOP =false;    
                            enable_walking();
                            break;
                  case '?': delay((rgroup -'0')*50+(rname-'A')*10);
                            Serial.print(" "); Serial.print(rgroup); Serial.print(rname); 
                            break;
                  case '.':  f_MsgForMe =0;  f_BeQuiet =1;  hashgroup =' ';       //end #session
                            break;
                  default:  if (Ccmnd>='1' && Ccmnd<='9' ) {
                              hashgroup =Ccmnd; 
                              if (hashgroup==rgroup) 
                                { f_MsgForMe =1;  f_BeQuiet =1;  }    //my group
                               else
                                { f_MsgForMe =0;  f_BeQuiet =1;  }    //not my group
                              break;
                              }
                            if (Ccmnd==rname && hashgroup==rgroup)
                              { f_MsgForMe =1;  f_BeQuiet =0;                 //me
                                if(hashgroup==0) { f_BeQuiet =1; }  break; }  //me and others
                             else
                              { f_MsgForMe =0;  f_BeQuiet =1;  break; }       //not me 
                            break;
                }
                break;      
SYSTEM_InitMind:
    case '*': if (f_recording) { break; }        //no action if recording
              GetCcmnd(); 
              if (Ccmnd=='i') { GetCcmnd(); if (Ccmnd=='M') { initialiseEEPROM(); } }
              if (Ccmnd=='r') { getnum(); Serial.print(EEPROM.read(behaveValue)); }          //getnum() sets behaveValue    // *r<EEadress>,                 //51
              if (Ccmnd=='w') { getnum(); EEadress =behaveValue; getnum(); EEPROM.write(EEadress,behaveValue); }  break;    // *w<EEadress>,<behaveValue>,   //51

    // end SYSTEM  
COMMAND_Default:                
    default:    f_recordC =false;    //don't record bad key		
                badkey();  if(!f_BeQuiet){ Serial.println(" bad key"); }  break;
  }    // end switch COMMAND
	
Record:	
  if (f_recording  && f_recordC) { RecordCcmnd(); } 
  
}
//== END {LOOP} =============================================

//== USER ROUTINES ==========================================
void Experiment()       //until Quit
{ while(true)
  { 
    /*/put your own routine here e.g.
    rollspeed =4;
    pacespeed =4;
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    forward();       // Walk one step forwards
    whee(); 
    turnRight();
    turnRight();
    turnRight();    
    whee(); 
    rollspeed =2;
    pacespeed =2;
    turnLeft();
    turnLeft();
    turnLeft();
    whee(); 
    rollspeed =1;
    pacespeed =1;
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    backward();      // Walk one step backwards
    whee();    
    whee();    
    whee();    */
    if(digitalRead(12)==0){whee();}
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------
void Xplore()         //Wander() until Quit
{ while(true)
  { Wander();
    if( random(9)>7 ) {chatter(); } 
    if(Quit()) { break; } 
  }	// end while
}
//----------------------------
void beAlive()         // the loop() from B_PM22_BeAlive
{ while(true)
  { switch (random(10))
    { case 0: WanderSteps(random(10)); break;    //uses all sensors
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
      case 9: chatter();
    } // end switch
    if( Quit() ) { break; }
  }	// end while
}

//== FUNCTIONS ==============================================
void waitNoIR()                                                     //50
{ 
testIR:
  if(digitalRead(IRinpin_)==0) { goto testIR; }
  delay(1);              //  maybe it was a 0 so test next byte
  if(digitalRead(IRinpin_)==0) { goto testIR; }
}
//----------------------------
void NewCcmnd()                                                       //50
{ oldCcmnd =Ccmnd;
  do { GetCcmnd(); }while(Ccmnd==oldCcmnd);
}
//----------------------------
void GetCcmnd()	       //reads Serial or playing Act until Ccmnd != 0, prints Ccmnd
{ Ccmnd =0;  f_IRcmnd =0;                                                   //50
  while(Ccmnd==0 && f_IRcmnd==0)                                            //50
  { if (f_recording)  
      { if (digitalRead(IRinpin_)==0) {    //pulses low if IR being received  //50 
          IRin(IRinpin_);             //sets f_IRcmnd if Ccmnd is '.' or '9'  //50
          //if (f_IRcmnd) { bip(); } }                                        //50
          if (Ccmnd!=0) { bip(); } }                                          //51
        if (Serial.available()) {
          Ccmnd =Serial.read();
          bip(); }
       }
      else         //if (!f_recording) 
       { if (digitalRead(IRinpin_)==0) {  //pulses low if IR being received     //50
           IRin(IRinpin_);                //sets f_IRcmnd if Ccmnd is '.' or '9'
          } 
         if (Serial.available()) {                                              //50 
           Ccmnd =Serial.read(); }  //over-ride Play if Serial, null ignored  
         if (!f_MsgForMe && Ccmnd!='#') {                                      //50  
           Ccmnd =0; f_IRcmnd =0; }     //discard messages for others          //50
         //if (Ccmnd!=0 || f_IRcmnd!=0) { 
         if (Ccmnd!=0) { 
            bip(); }    
          else {
           if (!f_hashcmnd) { //don't react or get a Play character if in the middle of a hash command   
             if (f_play || f_reacting) {
               Ccmnd =PlayAct(); }            //get Act or React Ccmnd 
               }  //end if (!f_hashcmnd)
           }   
       }  //end if (!f_recording)                                     
   }  //end while Ccmnd==0 && f_IRcmnd==0                                  //50          
  Serial.print(Ccmnd);
  //printCcmnd();
}   
//----------------------------
void printCcmnd()
{ if (!f_BeQuiet && !f_skipmind)
  { if (f_IRcmnd) {Serial.print("#"); Serial.print(byte(Ccmnd)); } else { Serial.print(Ccmnd); } }   //50
}
//----------------------------
void RecordCcmnd()	
{ EEPROM.write(EEPROMindex,Ccmnd);
  EEPROMindex++;
  if (EEPROMindex>=ActEnd) { f_recording =false;  EEPROM.write(ActEnd,'.');  badkey(); }     //terminated string 
}  
//----------------------------
void makeCcmndDigit()      //ensure 0 - 9
{ if (Ccmnd<'0' || Ccmnd>'9') { Ccmnd ='0'; } //default
}
//----------------------------  
void printMind()
{ PrintAllActs();
  printBehaviour();
}
//----------------------------
void PrintReActs()
{ Serial.println();  Serial.print("W & G, 1 indicates object or hole sensed LR");
  for(Act ='1'; Act<='8'; Act++) { epage =3; printReAct(); }  Serial.println(); 
}
//----------------------------
void printReAct()
{ Serial.println();
  Serial.print("ReAct");  Serial.print(Act);  Serial.print(" ");
  switch (Act)
  { case '1':  Serial.print("U<=");  Serial.print(SonarAwareAt);  Serial.print(" ");  break;
    case '2':  Serial.print("U<=");  Serial.print(SonarTooNear);  Serial.print(" ");  break;
    case '3':  Serial.print("W11");  Serial.print("  ");  break;
    case '4':  Serial.print("W10");  Serial.print("  ");  break;
    case '5':  Serial.print("W01");  Serial.print("  ");  break;
    case '6':  Serial.print("G11");  Serial.print("  ");  break;
    case '7':  Serial.print("G10");  Serial.print("  ");  break;
    case '8':  Serial.print("G01");  Serial.print("  ");  break;
  } // end switch
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);    //there are moves which take more than 1 character  
  Serial.print(" bytes"); //so the space taken in EEPROM is really bytes
  moves =0;	
}
//---------------------------- 
void loadBehaviour()

{ getBehaviour();  // 1 - 9   '.' quits
  Serial.print("Behaviour="); if(Behaviour=='9') { Serial.print("all"); } else { Serial.println(Behaviour); } 
  Serial.println("  , ; [Enter]   to enter numbers, [Esc] or '.' to quit");
  while( Ccmnd!='.' )    
  { Serial.println("enter - behaviour_number, behaviour_value,");
    do                                  //get BeHave address in the slot, ended with ','
    { Ccmnd =' ';
      if(Serial.available()) { while( Serial.peek()==CR || Serial.peek()==LF ) { Serial.read(); }}    //50.2  for send line CRLF terminals
      if(Serial.available()) { Ccmnd =Serial.read(); Serial.print(Ccmnd);  bip(); }
      if (isDigit(Ccmnd)) { numString += Ccmnd; Ccmnd =' '; }
       //[Return] is \r \n so let isDigit filter \r      
      }while( Ccmnd!='\n' && Ccmnd!=',' && Ccmnd!='.' && Ccmnd!=Esc ); 
      
    if(Ccmnd=='.' || Ccmnd==Esc) break;             //exit while

    behaveNum =numString.toInt(); numString ="";
 
    switch (behaveNum)
    { case 31:  //Ef_walking    - true, false
                get1char(); behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 30:  //ErollC        - ~90
                getnum();   behaveValue =constrain(behaveValue,80,100);  
                break;
      case 29:  //EpaceC        - ~90
                getnum();   behaveValue =constrain(behaveValue,80,100);  
                break;
      case 28:  //Erollby       - 1-15
                getnum();   behaveValue =constrain(behaveValue,1,maxroll);  
                break;
      case 27:  //Epaceby       - 1-15 
                getnum();   behaveValue =constrain(behaveValue,1,maxpace);  
                break;
      case 22:  //Ergroup       - '1-9'
                get1char();  behaveValue =constrain(Ccmnd,'1','9');
                break;
      case 21:  //Ername        - 'A-z'
                get1char();  behaveValue =constrain(Ccmnd,'A','z');
                break;
      case 20:  //EautoAct      - '0-8'
                get1char();  behaveValue =constrain(Ccmnd,'0','8');
                break;
      case 19:  //ESonarAwareAt - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 18:  //ESonarTooNear - 1-9
                get1char(); behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 17:  //Ef_cm         - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 16:  //Erollspeedmax - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 15:  //Epacespeedmax - 1-9
                getchar();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 14:  //Erollspeed    - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 13:  //Epacespeed    - 1-9
                get1char();  behaveValue =constrain(Ccmnd -'0',1,9);
                break;
      case 12:  //f_bG    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 11:  //f_bW    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 10:  //f_bU    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      case 9:   //f_AutoSense    - true, false
                get1char();  behaveValue =constrain(Ccmnd -'0',0,1);
                break;
      default:  behaveNum =0;
                Serial.print("bad BeHave number");  Serial.println();  
                break;      
      }   //end switch
    if(Ccmnd=='.' || Ccmnd==Esc) break;             //exit while  
    if(behaveNum!=0) { 
      if(Behaviour!='9') { 
        BehaviourEbase =(Behaviour -'1') *32; 
        EEPROM.write(BehaviourEbase +behaveNum, behaveValue);
       }   
       else {
         for ( Behaviour =0; Behaviour <8; Behaviour ++ )  {
           BehaviourEbase =Behaviour *32; 
           EEPROM.write(BehaviourEbase +behaveNum, behaveValue);
          }
         Behaviour ='9'; 
        }
      }  
    } //end while
}
//---------------------------- 
char get1char()
{ Ccmnd =0;
  do{ if(Serial.available())  Ccmnd =Serial.read(); }while(Ccmnd==0);
  Serial.println(Ccmnd);  bip();
}
//----------------------------  
byte getnum()
{ do
  { Ccmnd =' ';
    if(Serial.available()) { Ccmnd =Serial.read();  Serial.print(Ccmnd);  bip();} 
    if (isDigit(Ccmnd)) { numString +=Ccmnd;   Ccmnd =' '; }
       //[Return] is \r \n so let isDigit discard \r      
    }while(Ccmnd!='\n' && Ccmnd!='.' && Ccmnd!=',' && Ccmnd!=';' && Ccmnd!=Esc);
  behaveValue =numString.toInt();  numString = "";
  if(Ccmnd=='.' || Ccmnd==Esc) { Serial.println();  behaveNum =0; }
}
//----------------------------  
char getBehaviour()      //called when loading a behaviour, Ccmnd ->'.', '1' - '8'
{ Ccmnd =0;
  do
  { if(Serial.available()) 
    { Ccmnd =Serial.read(); bip();
      switch (Ccmnd) 
      { case Esc:  Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  break;           //quit
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':  Behaviour =Ccmnd;  break;
        default:   badkey(); Serial.print(" enter Behaviour or '.'"); Ccmnd =0;
       }  //end case
     } //end if Serial
   } while(Ccmnd==0);
  Serial.println(Ccmnd);
}    
//----------------------------  
void printBehaviour()
{ Serial.println();  Serial.println();
  Serial.println(thisprog); // so we know which program is running
  Serial.print(rgroup);  Serial.print(rname); Serial.println(" robot");     
  Serial.print(f_MsgForMe);    Serial.println(" MsgForMe");  
  Serial.print(f_BeQuiet);     Serial.println(" BeQuiet");
  Serial.print(Behaviour);     Serial.println(" Behaviour"); 
  Serial.print(f_walking);     Serial.println(" walking"); 
  Serial.print(autoAct);       Serial.print(" autoAct,  "); Serial.print("rerun "); 
      if (f_runDefault==1) { Serial.print("On"); } else { Serial.print("Off"); } Serial.println("  (default Off)"); 
  Serial.print(rollspeed);     Serial.println(" rollspeed");
  Serial.print(pacespeed);     Serial.println(" pacespeed");
  Serial.print(rollspeedmax);  Serial.println(" rollspeedmax");
  Serial.print(pacespeedmax);  Serial.println(" pacespeedmax");  
  Serial.print(rollby);        Serial.print('(');  Serial.print(rollby/rollbyScale);  Serial.print(')');  Serial.println(" rollby");
  Serial.print(paceby);        Serial.print('(');  Serial.print(paceby/pacebyScale);  Serial.print(')');  Serial.println(" paceby");
  Serial.print(turnby);        Serial.print('(');  Serial.print(turnby/turnbyScale);  Serial.print(')');  Serial.println(" turnby");
  Serial.print(rollC);         Serial.println(" rollC");    
  Serial.print(paceC);         Serial.println(" paceC");
  Serial.print(SonarAwareAt);  Serial.println(" SonarAwareAt");   
  Serial.print(SonarTooNear);  Serial.println(" SonarTooNear");   
  Serial.print(f_bG);          Serial.println(" Greact");   
  Serial.print(f_bW);          Serial.println(" Wreact");   
  Serial.print(f_bU);          Serial.println(" Ureact");   
  Serial.print(f_bAutoSense);  Serial.println(" auto$ense");
  Serial.print(f_bv);          Serial.println(" verbose");
//disp_freeram();  
}
//----------------------------  
void PrintBehaviours()
//This uses Serial.print(F(" text ")); which saves RAM by leaving the text in Program memory 
//and is a lot easier to use than the PROGMEM construct previously used for the help text - we can learn something every day!
{ Serial.println();
  Serial.println(F("BEHAVIOUR       b1    b2    b3    b4    b5    b6    b7    b8"));
  Serial.println(F(" # BeHave       --    --    --    --    --    --    --    --"));
  Serial.print(F("31 f_walking    "));
    for(int i =31; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0=false, 1=true");
  Serial.print(F("30 rollC        "));
    for(int i =30; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  Serial.print(F("29 paceC        "));
    for(int i =29; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.println("servo degrees");
  Serial.print(F("28 rollby       "));
    for(int i =28; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.print("degrees =cmnd *"); Serial.println(rollbyScale);
  Serial.print(F("27 paceby       "));
    for(int i =27; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("    ")); } Serial.print("degrees =cmnd *"); Serial.println(pacebyScale);
  //23-26 never altered
  Serial.print(F("22 rgroup       "));
    for(int i =22; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("21 rname        "));
    for(int i =21; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("A-z");
  Serial.print(F("20 autoAct      "));
    for(int i =20; i<256; i +=32) { Serial.print(char(EEPROM.read(i))); Serial.print(F("     ")); } Serial.println("0-8");
  Serial.print(F("19 SonarAwareAt "));
    for(int i =19; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0-9, units 5cm or 2inch");
  Serial.print(F("18 SonarTooNear "));
    for(int i =18; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0-9, units 5cm or 2inch");
  Serial.print(F("17 Sonar use cm "));
    for(int i =17; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0=false, 1=true");
  Serial.print(F("16 rollspeedmax "));
    for(int i =16; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("15 pacespeedmax "));
    for(int i =15; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("14 rollspeed    "));
    for(int i =14; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("13 pacespeed    "));
    for(int i =13; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("1-9");
  Serial.print(F("12 Greact       "));
    for(int i =12; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0,1");
  Serial.print(F("11 Wreact       ")); 
    for(int i =11; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0,1");
  Serial.print(F("10 Ureact       "));   
    for(int i =10; i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0,1");
  Serial.print(F(" 9 auto$ense    "));   
    for(int i =9;  i<256; i +=32) { Serial.print(EEPROM.read(i));       Serial.print(F("     ")); } Serial.println("0,1");
  Serial.print(F("-- @Behaviour   "));
    for(int i =0; i<256; i +=32) { Serial.print(char(EEPROM.read(i)));  Serial.print(F("     ")); } Serial.println();
} 
//----------------------------  
void DumpActs()        //Dump Acts, Acts0, ReActs. Print without names and length
{ for(Act ='1'; Act<='8'; Act++)
  { setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =2;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  for(Act ='1'; Act<='8'; Act++) 
  { epage =3;  setEEPROMindexes(Act);
    Serial.println();
    Ccmnd =EEPROM.read(EEPROMindex++);
    // -1 (=255) is uninitialised EEPROM value
    while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); Ccmnd =EEPROM.read(EEPROMindex++); }
  }
  Serial.println();
}		
//----------------------------  
void loadAct()
{ redLEDon();  
  getAct();                         // Ccmnd ->'.' - Quit,
                                    // '0', '!'    - sets Acts0 or ReActs page then gets Act
                                    // '1' - '8'   - sets Act '1' - '8',  - 
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    while(true) 
    { while(!Serial.available()) {}  //wait for Serial
      Ccmnd =Serial.read();  Serial.print(Ccmnd);
      if(Ccmnd=='.' || Ccmnd==Esc) { break; }
      EEPROM.write(EEPROMindex,Ccmnd);  EEPROMindex++;
      if (EEPROMindex>=ActEnd) { badkey();  break; }     //ActEnd =EEPROMindex +31;   
    }
    EEPROM.write(EEPROMindex,'.');                        //terminated string 
  }
}
//----------------------------  
void copyAct()
{ getAct();
  if (Ccmnd!='.')
  { setEEPROMindexes(Act);
    int fromEEPROMindex =EEPROMindex;  
    int fromActEnd =ActEnd;
    getAct();
    if (Ccmnd!='.')
    { setEEPROMindexes(Act);
      for(byte i =0; i<=fromActEnd -fromEEPROMindex; i++)
      { EEPROM.write(EEPROMindex +i,EEPROM.read(fromEEPROMindex +i)); }
    } 
  }	
}
//----------------------------
char PlayAct() 
{ char _Ccmnd =EEPROM.read(EEPROMindex++);
  if(_Ccmnd==255 || _Ccmnd==-1) _Ccmnd ='.';     //uninitialised EEPROM value, don't play it, change to '.'
  return _Ccmnd;
}  
//----------------------------
void PrintAllActs()
{ for(Act ='1'; Act<='8'; Act++) { epage =1; printAct(); }  Serial.println(); 
  for(Act ='1'; Act<='8'; Act++) { epage =2; printAct(); }  Serial.println(); 
}
//----------------------------
void printAct()
{ Serial.println();
  Serial.print("Act");
  if(epage==2) { Serial.print('0'); }    //for logical page0 Acts
  Serial.print(Act);
  Serial.print(" ");
  setEEPROMindexes(Act);
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);
  Serial.print(" bytes");
  moves =0;	
}
//----------------------------  
void getAct()      //called when recording, loading, printing and copying Acts, Ccmnd ->'.', '1' - '8'
{ Ccmnd =0;
  do
  { //if(Serial.available())                                   //51
    //{ //Ccmnd =Serial.read();  Serial.print(Ccmnd);          //51
      GetCcmnd();                                              //51
      switch (Ccmnd) 
      { case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  Serial.print(">");  break;     //quit, print ready prompt
        case '!':  epage =3;  Ccmnd =0;  break;  //do it again to read Act
        case '0':  epage =2;  Ccmnd =0;  break;  //do it again to read Act
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':  Act =Ccmnd;  break;
        default: badkey(); Serial.print(" enter Act or '.'"); Ccmnd =0;	
      }  //end case
    //} //end if Serial                                        //51
  } while(Ccmnd==0);
}    
//----------------------------
void setEEPROMindexes(char Act)     // set EEPROM pointers
{ actIndex =Act -'1';               // ASCII to numeric,  Act 1 == actIndex 0
  EEPROMindex =(256 *epage) +(actIndex*32);  
  ActEnd =EEPROMindex +31;
  epage =1;            //set to default page, Acts 1 - 8 
}
//----------------------------
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF  
  whoa();  
}  
//----------------------------
void USreact()     // "U" read Ultrasonic sensor and react to objectRange
{ readUSdistance();
  if (f_bU) { reactUObjects(); }
}
//----------------------------
void WhiskerReact()     // "W" read Whisker sensors and avoid obstacles (if any) using avoidObstacles() behaviour rules
{ 
#ifdef BambinoMM
  return;
#endif  
  readWhiskers();
  queryObstaclesLR();
  if (f_bW) { reactObstaclesLR(); }
}
//----------------------------
void BeardReact()     // "G" Ground - read Beard sensors and avoid holes (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdgeLR();
  if (f_bG) { reactHolesLR(); }
}
//---------------------------
boolean Quit()          //check for Quit  //discard CR LF and check IRpin_ - 50.1
{ if( Serial.available() ) { while( Serial.peek()==CR || Serial.peek()==LF ) { Serial.read(); } }
  if( Serial.available() || digitalRead(IRinpin_)==0 ) { return true; }  else  { return false; }  
}
//----------------------------
void bip()            // make a bip sound
{ beep(voicePin,1500,50); }
//----------------------------
#ifdef BambinoMM
void redLEDdim()
{ analogWrite(redLEDpin_,254); }  //redLED, LOW=on so 255 is off 
//----------------------------
void redLEDon()
{ digitalWrite(redLEDpin_,LOW); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin_,HIGH); }  
//---------------------------- 
//BambinoMM does not have Whisker sensors
//---------------------------- 
#else
void redLEDon()
{ digitalWrite(redLEDpin,HIGH); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin,LOW); }  
//----------------------------
void whiskerLEDsOff()
{ digitalWrite(RwhiskerLEDpin_, HIGH);     // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);     // whisker LED OFF
  }  
//----------------------------  
void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
  }
#endif
//----------------------------  
void beardLEDsOff()
{ digitalWrite(RbeardLEDpin_, HIGH);       // beard LED Off
	digitalWrite(LbeardLEDpin_, HIGH);       // beard LED Off
  }
//------------------------------
#ifndef BambinoMM
void IRout()  {                 //52
/*
 do { 
  digitalWrite(8,HIGH);
  delayMicroseconds(10); 
  digitalWrite(8,LOW);
  delayMicroseconds(11);     
  } while (true);
*/

//PicAxe IR is 12 bits taking 18ms with a gap of 25ms and three bytes are sent
  
  //40KHz  25 us per cycles
  //header 2400us - 2400 / 25 = 96 cycles
  //one    1200us - 1200 / 25 = 48 
  //gap     600us -  600 / 25 = 24 cycles
  //zero    600us -  600 / 25 = 24 cycles
  
  //38KHz  26.315789 us per cycles
  //header 2400us - 2400 / 26.3 = 91 cycles
  //one    1200us - 1200 / 26.3 = 46 
  //gap     600us -  600 / 26.3 = 23 cycles
  //zero    600us -  600 / 26.3 = 23 cycles

byte IRcmnd;  
  IRcmnd =translate2IR();
  
  for ( byte i=0; i<3; i++ ) {
IR_header:
    IR_on(91,1);
IR_data:  
    for ( byte i=0; i<7; i++ ) {
      IR_on(23,0);
      if( bitRead(IRcmnd, i)) { IR_on(46,1); } else {IR_on(23,1); }
     }
    IR_on(23,0);    //now 5 bit address =1 for PicAxe, ignored by Bamduino
    IR_on(46,1);
    IR_on(23,0);
    IR_on(23,1);
    IR_on(23,0);
    IR_on(23,1);
    IR_on(23,0);
    IR_on(23,1);
    IR_on(23,0);
    IR_on(23,1);
    delay(30);
   }
  delay(100);  
}
//------------------------------
void IR_on(byte cycles, boolean state ) {
  //11 - 37.24
  //10,11 - 38.68  
  //10 - 40.25       using 40KHz
  for ( byte i=1; i<=cycles; i++ ) {
    digitalWrite(IRoutpin,state);
    delayMicroseconds(10); 
    digitalWrite(IRoutpin,LOW);
    delayMicroseconds(11);    
  }
}
//------------------------------
byte translate2IR() {                       //52
  //Serial.print(byte(Ccmnd));
byte IRcmnd;  
  switch (Ccmnd)
  { case '.':  IRcmnd =keyPOWER;  break;
    case 'F':  IRcmnd =keyUP;     break;
    case 'B':  IRcmnd =keyDOWN;   break;
    case 'R':  IRcmnd =keyRIGHT;  break;    // [>]
    case 'L':  IRcmnd =keyLEFT;   break;    // [<]
    case 'r':  IRcmnd =keyBAR;    break;    // [f|]
    case 'a':  IRcmnd =keyTENT;   break;    // [f^]
    //case ''keyVCROSS: IRcmnd ='?';  break;    // [f+]
    case '#':  IRcmnd =keyXCROSS; break;    // [fx]
    case '1':  IRcmnd =key1;      break;
    case '2':  IRcmnd =key2;      break;
    case '3':  IRcmnd =key3;      break;
    case '4':  IRcmnd =key4;      break;
    case '5':  IRcmnd =key5;      break;
    case '6':  IRcmnd =key6;      break;
    case '7':  IRcmnd =key7;      break;
    case '8':  IRcmnd =key8;      break;
    case '0':  IRcmnd =key0;      break;
    case '9':  IRcmnd =key9;      break;
    case '-':  IRcmnd =keyMINUS;  break;
    case '+':  IRcmnd =keyPLUS;   break;
    default:   IRcmnd =0;         break;    
  } 
  return IRcmnd;
}
#endif
//------------------------------
void IRin(byte pin)              //f_IRcmnd==1 if Ccmnd ='.' or '9'     //50
{ //Serial.print("IRin ");
  
//header =2400us;  //2418
//1 =1200us
//0 =600us
//each bit preceded by 600us off
  byte inbit;
  int pulsetime;
  long timeout;
  
  timeout =millis() +500;                                     //51
  f_IRcmnd =0;  
IRin_WaitHeader:
  while( digitalRead(pin)==1 && millis()<=timeout) {}    
  if (millis()>timeout) { Serial.print("xIR"); redLEDoff();  goto IRin_done; }
IRin_Header: 
  pulsetime =pulseIn(pin,LOW,50000);                //50 ms timeout
  if (pulsetime<2200) { redLEDon();  goto IRin_WaitHeader; }
IRin_Message:
  for(inbit=0; inbit<7; inbit++) {
    pulsetime =pulseIn(pin,LOW);
    if (pulsetime<1000) { bitWrite(Ccmnd, inbit, 0); } else { bitWrite(Ccmnd, inbit, 1); }
  }
  IRtranslate();
  delay(100);                         //wait finger off    //51
IRin_done: ;
}  
//------------------------------
void IRtranslate() {
  //Serial.print(byte(Ccmnd));
  switch (Ccmnd)
  { case keyPOWER:  Ccmnd ='.';  f_IRcmnd =1;  break;
    case keyUP:     Ccmnd ='F';  break;
    case keyDOWN:   Ccmnd ='B';  break;
    case keyRIGHT:  Ccmnd ='R';  if(f_hashcmnd) { Ccmnd ='>'; }  break;    // [>]
    case keyLEFT:   Ccmnd ='L';  if(f_hashcmnd) { Ccmnd ='<'; }  break;    // [<]
    case keyBAR:    Ccmnd ='r';  break;    // [f|]
    case keyTENT:   Ccmnd ='a';  break;    // [f^]
    case keyVCROSS: Ccmnd ='?';  break;    // [f+]
    case keyXCROSS: Ccmnd ='#';  break;    // [fx]
    case key1:      Ccmnd ='1';  break;
    case key2:      Ccmnd ='2';  break;
    case key3:      Ccmnd ='3';  break;
    case key4:      Ccmnd ='4';  break;
    case key5:      Ccmnd ='5';  break;
    case key6:      Ccmnd ='6';  break;
    case key7:      Ccmnd ='7';  break;
    case key8:      Ccmnd ='8';  break;
    case key0:      Ccmnd ='0';  break;
    case key9:      Ccmnd ='9';  f_IRcmnd =1;  break;
    case keyMINUS:  Ccmnd ='-';  break;
    case keyPLUS:   Ccmnd ='+';  break;
    default:        Ccmnd =0;    break;
  } 
}
//===========================================================
/*****************Dan*************************/
// print integer value. 
/*void i_printf( int val ) {  Serial.print(val);  }

// print string s1. 
void s_printf( char *s1 ) {  Serial.print(s1);  }
  
/*********************************************/
/*void disp_freeram()
{
  s_printf("-SRAM left: ");
  i_printf( freeRam() );
}
/*********************************************/
/*int freeRam() 
{
  extern int __heap_start,*__brkval;
  int v;
  return (int)&v - (__brkval == 0  
    ? (int)&__heap_start : (int) __brkval);  
}  
/*********************************************/
